@@ -16,13 +16,14 @@
 import logging
 from abc import ABC
 from functools import lru_cache
-from typing import Any, Dict
+from typing import Dict
 
 import torch
 from torch.nn import Linear, ModuleList
 
+from ludwig.api_annotations import DeveloperAPI
 from ludwig.constants import BINARY, NUMBER
-from ludwig.encoders.registry import sequence_encoder_registry
+from ludwig.encoders.registry import get_sequence_encoder_registry
 from ludwig.features.base_feature import InputFeature
 from ludwig.modules.attention_modules import TransformerStack
 from ludwig.modules.embedding_modules import Embed
@@ -37,7 +38,7 @@ from ludwig.schema.combiners.sequence_concat import SequenceConcatCombinerConfig
 from ludwig.schema.combiners.tab_transformer import TabTransformerCombinerConfig
 from ludwig.schema.combiners.tabnet import TabNetCombinerConfig
 from ludwig.schema.combiners.transformer import TransformerCombinerConfig
-from ludwig.schema.combiners.utils import combiner_registry, get_combiner_jsonschema, register_combiner
+from ludwig.schema.combiners.utils import combiner_registry, register_combiner
 from ludwig.utils.misc_utils import get_from_registry
 from ludwig.utils.torch_utils import LudwigModule, sequence_length_3D
 from ludwig.utils.torch_utils import sequence_mask as torch_sequence_mask
@@ -54,12 +55,15 @@ def get_combiner_class(combiner_type: str):
     return get_from_registry(combiner_type, combiner_registry)
 
 
-def get_combiner_schema() -> Dict[str, Any]:
-    return get_combiner_jsonschema()
-
-
-# super class to house common properties
+@DeveloperAPI
 class Combiner(LudwigModule, ABC):
+    """Base class for combiners, which implements common properties.
+
+    Subclasses will usually override:     __init__()        to set properties and allocate resources.  Should call
+    super().__init__(input_features).     forward()         performs the forward pass given a dictionary of encoder
+    outputs.     get_schema_cls()  must returns the class of the corresponding schema for the combiner type.
+    """
+
     def __init__(self, input_features: Dict[str, "InputFeature"]):
         super().__init__()
         self.input_features = input_features
@@ -104,28 +108,27 @@ class ConcatCombiner(Combiner):
 
         # todo future: this may be redundant, check
         fc_layers = config.fc_layers
-        if fc_layers is None and config.num_fc_layers is not None:
+        if fc_layers is None:
             fc_layers = []
             for i in range(config.num_fc_layers):
                 fc_layers.append({"output_size": config.output_size})
-
         self.fc_layers = fc_layers
-        if self.fc_layers is not None:
-            logger.debug("  FCStack")
-            self.fc_stack = FCStack(
-                first_layer_input_size=self.concatenated_shape[-1],
-                layers=config.fc_layers,
-                num_layers=config.num_fc_layers,
-                default_output_size=config.output_size,
-                default_use_bias=config.use_bias,
-                default_weights_initializer=config.weights_initializer,
-                default_bias_initializer=config.bias_initializer,
-                default_norm=config.norm,
-                default_norm_params=config.norm_params,
-                default_activation=config.activation,
-                default_dropout=config.dropout,
-                residual=config.residual,
-            )
+
+        logger.debug("  FCStack")
+        self.fc_stack = FCStack(
+            first_layer_input_size=self.concatenated_shape[-1],
+            layers=config.fc_layers,
+            num_layers=config.num_fc_layers,
+            default_output_size=config.output_size,
+            default_use_bias=config.use_bias,
+            default_weights_initializer=config.weights_initializer,
+            default_bias_initializer=config.bias_initializer,
+            default_norm=config.norm,
+            default_norm_params=config.norm_params,
+            default_activation=config.activation,
+            default_dropout=config.dropout,
+            residual=config.residual,
+        )
 
         if input_features and len(input_features) == 1 and self.fc_layers is None:
             self.supports_masking = True
@@ -145,8 +148,7 @@ class ConcatCombiner(Combiner):
             hidden = list(encoder_outputs)[0]
 
         # ================ Fully Connected ================
-        if self.fc_stack is not None:
-            hidden = self.fc_stack(hidden)
+        hidden = self.fc_stack(hidden)
 
         return_data = {"combiner_output": hidden}
 
@@ -198,8 +200,6 @@ class SequenceConcatCombiner(Combiner):
             if len(self.input_features[k].output_shape) == 2:
                 seq_size = self.input_features[k].output_shape[0]
                 break
-        if not seq_size:
-            raise ValueError("At least one of the input features for SequenceConcatCombiner should be a sequence.")
 
         # collect the size of the last dimension for all input feature
         # encoder outputs
@@ -291,7 +291,6 @@ class SequenceConcatCombiner(Combiner):
         logger.debug(f"  concat_hidden: {hidden}")
 
         # ================ Mask ================
-        # todo future: maybe modify this with TF2 mask mechanics
         sequence_mask = torch_sequence_mask(sequence_length, sequence_max_length)
         hidden = torch.multiply(hidden, torch.unsqueeze(sequence_mask, -1).type(torch.float32))
 
@@ -328,7 +327,7 @@ class SequenceCombiner(Combiner):
             f"combiner input shape {self.combiner.concatenated_shape}, " f"output shape {self.combiner.output_shape}"
         )
 
-        self.encoder_obj = get_from_registry(config.encoder, sequence_encoder_registry)(
+        self.encoder_obj = get_from_registry(config.encoder.type, get_sequence_encoder_registry())(
             should_embed=False,
             reduce_output=config.reduce_output,
             embedding_size=self.combiner.output_shape[1],
@@ -566,8 +565,6 @@ class TabTransformerCombiner(Combiner):
         self.name = "TabTransformerCombiner"
         logger.debug(f"Initializing {self.name}")
 
-        if config.reduce_output is None:
-            raise ValueError("TabTransformer requires the `reduce_output` " "parameter")
         self.reduce_output = config.reduce_output
         self.reduce_sequence = SequenceReducer(
             reduce_mode=config.reduce_output, max_sequence_length=len(input_features), encoding_size=config.hidden_size
