@@ -39,13 +39,11 @@ from .aim import AimCallback
 from .comet import CometCallback
 from .mlflow import MlflowCallback
 from .wandb import WandbCallback
-from .whylogs import WhyLogsCallback
 
 contrib_registry = {
     # Contributors, add your class here:
     "comet": CometCallback,
     "wandb": WandbCallback,
     "mlflow": MlflowCallback,
-    "whylogs": WhyLogsCallback,
     "aim": AimCallback,
 }
