@@ -1,6 +1,7 @@
 import json
 import logging
 
+from ludwig.api_annotations import PublicAPI
 from ludwig.callbacks import Callback
 from ludwig.utils.data_utils import NumpyEncoder
 from ludwig.utils.package_utils import LazyLoader
@@ -10,6 +11,7 @@ aim = LazyLoader("aim", globals(), "aim")
 logger = logging.getLogger(__name__)
 
 
+@PublicAPI
 class AimCallback(Callback):
     """Class that defines the methods necessary to hook into process."""
 
@@ -51,7 +53,7 @@ class AimCallback(Callback):
 
         if self.aim_run:
             for key, value in progress_tracker.log_metrics().items():
-                if "metrics" in key:
+                if "metrics" in key and "best" not in key:
                     metrics_dict_name, feature_name, metric_name = key.split(".")
 
                     self.aim_run.track(
