@@ -14,28 +14,49 @@
 # limitations under the License.
 # ==============================================================================
 
+import collections
 import logging
-from typing import Dict
+from contextlib import contextmanager
+from typing import Any, Dict, Iterable, Tuple, Union
 
 import dask
 import dask.array as da
 import dask.dataframe as dd
+import numpy as np
+import pandas as pd
+import pyarrow as pa
+import ray
 from dask.diagnostics import ProgressBar
+from packaging import version
+from pyarrow.fs import FSSpecHandler, PyFileSystem
+from ray.data import Dataset, read_parquet
+from ray.data.block import Block, BlockAccessor
+from ray.data.extensions import ArrowTensorType, TensorDtype
+from ray.util.client.common import ClientObjectRef
 
+from ludwig.api_annotations import DeveloperAPI
 from ludwig.data.dataframe.base import DataFrameEngine
-from ludwig.utils.data_utils import get_pa_schema, split_by_slices
+from ludwig.utils.data_utils import get_pa_schema, get_parquet_filename, split_by_slices
 from ludwig.utils.dataframe_utils import set_index_name
+from ludwig.utils.fs_utils import get_fs_and_path
 
 TMP_COLUMN = "__TMP_COLUMN__"
 
+# This is to be compatible with pyarrow.lib.schema
+PandasBlockSchema = collections.namedtuple("PandasBlockSchema", ["names", "types"])
 
 logger = logging.getLogger(__name__)
 
 
+_ray_230 = version.parse(ray.__version__) >= version.parse("2.3.0")
+
+
+@DeveloperAPI
 def set_scheduler(scheduler):
     dask.config.set(scheduler=scheduler)
 
 
+@DeveloperAPI
 def reset_index_across_all_partitions(df):
     """Compute a monotonically increasing index across all partitions.
 
@@ -54,6 +75,7 @@ def reset_index_across_all_partitions(df):
     return df
 
 
+@DeveloperAPI
 class DaskEngine(DataFrameEngine):
     def __init__(self, parallelism=None, persist=True, _use_ray=True, **kwargs):
         from ray.util.dask import ray_dask_get
@@ -137,12 +159,22 @@ class DaskEngine(DataFrameEngine):
         meta = meta if meta is not None else ("data", "object")
         return series.map_partitions(map_fn, meta=meta)
 
-    def map_batches(self, series, map_fn):
+    def map_batches(self, series, map_fn, enable_tensor_extension_casting=True):
+        """Map a function over batches of a Dask Series.
+
+        Args:
+            series: Dask Series
+            map_fn: Function to apply to each batch
+            enable_tensor_extension_casting: Whether to enable tensor extension casting at the end of the Ray Datasets
+                map_batches call. This is useful in cases where the output is not supported by the ray Tensor dtype
+                extension, such as when the output consists of ragged tensors.
+        """
         import ray.data
 
-        ds = ray.data.from_dask(series)
-        ds = ds.map_batches(map_fn, batch_format="pandas")
-        return ds.to_dask()
+        with tensor_extension_casting(enable_tensor_extension_casting):
+            ds = ray.data.from_dask(series)
+            ds = ds.map_batches(map_fn, batch_format="pandas")
+            return self._to_dask(ds)
 
     def apply_objects(self, df, apply_fn, meta=None):
         meta = meta if meta is not None else ("data", "object")
@@ -193,19 +225,93 @@ class DaskEngine(DataFrameEngine):
                 engine="pyarrow",
                 write_index=index,
                 schema=schema,
+                name_function=get_parquet_filename,
             )
 
-    def to_ray_dataset(self, df):
+    def write_predictions(self, df: dd.DataFrame, path: str):
+        ds = self.to_ray_dataset(df)
+        # We disable tensor extension casting here because we are writing out to Parquet and there is no need
+        # to cast to the ray Tensor dtype extension before doing so (they will be written out as object dtype as if
+        # we were writing to parquet using dask).
+        with tensor_extension_casting(False):
+            fs, path = get_fs_and_path(path)
+            ds.write_parquet(path, filesystem=PyFileSystem(FSSpecHandler(fs)))
+
+    def read_predictions(self, path: str) -> dd.DataFrame:
+        fs, path = get_fs_and_path(path)
+        ds = read_parquet(path, filesystem=PyFileSystem(FSSpecHandler(fs)))
+        return self.from_ray_dataset(ds)
+
+    def to_ray_dataset(self, df) -> Dataset:
         from ray.data import from_dask
 
         return from_dask(df)
 
     def from_ray_dataset(self, dataset) -> dd.DataFrame:
-        return dataset.to_dask()
+        return self._to_dask(dataset)
 
     def reset_index(self, df):
         return reset_index_across_all_partitions(df)
 
+    def _to_dask(
+        self,
+        dataset: Dataset,
+        meta: Union[
+            pd.DataFrame,
+            pd.Series,
+            Dict[str, Any],
+            Iterable[Any],
+            Tuple[Any],
+            None,
+        ] = None,
+    ) -> dd.DataFrame:
+        """Custom Ray to_dask() conversion implementation with meta inference added for compatibility with Ray 2.0
+        and Ray 2.1. Useful for Ray Datasets that have image and audio features.
+
+        TODO(Arnav): Remove in Ray 2.2
+        """
+        if _ray_230:
+            return dataset.to_dask()
+
+        @dask.delayed
+        def block_to_df(block: Block):
+            if isinstance(block, (ray.ObjectRef, ClientObjectRef)):
+                raise ValueError(
+                    "Dataset.to_dask() must be used with Dask-on-Ray, please "
+                    "set the Dask scheduler to ray_dask_get (located in "
+                    "ray.util.dask)."
+                )
+            block = BlockAccessor.for_block(block)
+            return block.to_pandas()
+
+        # Infer Dask metadata from Datasets schema.
+        schema = dataset.schema()
+        if isinstance(schema, PandasBlockSchema):
+            meta = pd.DataFrame(
+                {
+                    col: pd.Series(dtype=(dtype if not isinstance(dtype, TensorDtype) else np.object_))
+                    for col, dtype in zip(schema.names, schema.types)
+                }
+            )
+        elif isinstance(schema, pa.Schema):
+            if any(isinstance(type_, ArrowTensorType) for type_ in schema.types):
+                meta = pd.DataFrame(
+                    {
+                        col: pd.Series(
+                            dtype=(dtype.to_pandas_dtype() if not isinstance(dtype, ArrowTensorType) else np.object_)
+                        )
+                        for col, dtype in zip(schema.names, schema.types)
+                    }
+                )
+            else:
+                meta = schema.empty_table().to_pandas()
+
+        ddf = dd.from_delayed(
+            [block_to_df(block) for block in dataset.get_internal_block_refs()],
+            meta=meta,
+        )
+        return ddf
+
     @property
     def array_lib(self):
         return da
@@ -221,3 +327,25 @@ class DaskEngine(DataFrameEngine):
     @property
     def partitioned(self):
         return True
+
+
+@contextmanager
+def tensor_extension_casting(enforced: bool):
+    """This context manager is used to enforce or disable tensor extension casting.
+
+    Ray Datasets will automatically cast tensor columns to the ray Tensor dtype extension at the end of
+    map_batches calls and before writing to Parquet. This context manager can be used to disable this behavior
+    and keep the tensor columns as object dtype. This is useful for writing to Parquet using dask.
+
+    Args:
+        enforced (bool): Whether to enforce tensor extension casting.
+    """
+    from ray.data.context import DatasetContext
+
+    ctx = DatasetContext.get_current()
+    prev_enable_tensor_extension_casting = ctx.enable_tensor_extension_casting
+    try:
+        ctx.enable_tensor_extension_casting = enforced
+        yield
+    finally:
+        ctx.enable_tensor_extension_casting = prev_enable_tensor_extension_casting
