@@ -1,5 +1,4 @@
 #! /usr/bin/env python
-# coding=utf-8
 # Copyright (c) 2020 Uber Technologies, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -19,33 +18,42 @@ import math
 
 import numpy as np
 
+from ludwig.distributed import DistributedStrategy
+from ludwig.utils.defaults import default_random_seed
+
 
 class DistributedSampler:
     """Adapted from `torch.utils.data.distributed.DistributedSampler`."""
-    def __init__(self, dataset_size, shuffle=True, seed=0, horovod=None):
+
+    def __init__(
+        self,
+        dataset_size: int,
+        shuffle: bool = True,
+        random_seed: int = default_random_seed,
+        distributed: DistributedStrategy = None,
+    ):
         self.dataset_size = dataset_size
-        self.num_replicas = horovod.size() if horovod else 1
-        self.rank = horovod.rank() if horovod else 0
+        self.num_replicas = distributed.size() if distributed else 1
+        self.rank = distributed.rank() if distributed else 0
         self.epoch = 0
         self.num_samples = int(math.ceil(self.dataset_size * 1.0 / self.num_replicas))
         self.total_size = self.num_samples * self.num_replicas
         self.shuffle = shuffle
-        self.seed = seed
+        self.random_seed = random_seed
 
     def __iter__(self):
         if self.shuffle:
             # deterministically shuffle based on epoch and seed
-            indices = np.random.RandomState(seed=self.seed + self.epoch)\
-                .permutation(self.dataset_size).tolist()
+            indices = np.random.RandomState(seed=self.random_seed + self.epoch).permutation(self.dataset_size).tolist()
         else:
             indices = list(range(self.dataset_size))
 
         # add extra samples to make it evenly divisible
-        indices += indices[:(self.total_size - len(indices))]
+        indices += indices[: (self.total_size - len(indices))]
         assert len(indices) == self.total_size
 
         # subsample
-        indices = indices[self.rank:self.total_size:self.num_replicas]
+        indices = indices[self.rank : self.total_size : self.num_replicas]
         assert len(indices) == self.num_samples
 
         return iter(indices)
