@@ -19,6 +19,7 @@ from typing import Optional
 import torch
 from torch import nn
 
+from ludwig.api_annotations import DeveloperAPI
 from ludwig.constants import AUDIO, SEQUENCE, TEXT, TIMESERIES
 from ludwig.encoders.base import Encoder
 from ludwig.encoders.registry import register_encoder, register_sequence_encoder
@@ -42,8 +43,13 @@ from ludwig.schema.encoders.sequence_encoders import (
 logger = logging.getLogger(__name__)
 
 
+class SequenceEncoder(Encoder):
+    pass
+
+
+@DeveloperAPI
 @register_encoder("passthrough", [SEQUENCE, TEXT, TIMESERIES])
-class SequencePassthroughEncoder(Encoder):
+class SequencePassthroughEncoder(SequenceEncoder):
     def __init__(
         self,
         reduce_output: str = None,
@@ -97,8 +103,9 @@ class SequencePassthroughEncoder(Encoder):
         return SequencePassthroughConfig
 
 
+@DeveloperAPI
 @register_encoder("embed", [SEQUENCE, TEXT])
-class SequenceEmbedEncoder(Encoder):
+class SequenceEmbedEncoder(SequenceEncoder):
     def __init__(
         self,
         vocab,
@@ -168,7 +175,7 @@ class SequenceEmbedEncoder(Encoder):
                 other keys for its parameters, e.g.
                 `{type: normal, mean: 0, stddev: 0}`.
                 To know the parameters of each initializer, please refer to
-                TensorFlow's documentation.
+                PyTorch's documentation.
         :type weights_initializer: str
         :param dropout: Tensor (torch.float) The dropout probability.
         :type dropout: Tensor
@@ -234,9 +241,10 @@ class SequenceEmbedEncoder(Encoder):
         return self.reduce_sequence.output_shape
 
 
+@DeveloperAPI
 @register_sequence_encoder("parallel_cnn")
 @register_encoder("parallel_cnn", [AUDIO, SEQUENCE, TEXT, TIMESERIES])
-class ParallelCNN(Encoder):
+class ParallelCNN(SequenceEncoder):
     def __init__(
         self,
         should_embed=True,
@@ -387,7 +395,7 @@ class ParallelCNN(Encoder):
                other keys for its parameters,
                e.g. `{type: normal, mean: 0, stddev: 0}`.
                To know the parameters of each initializer, please refer
-               to TensorFlow's documentation.
+               to PyTorch's documentation.
         :type initializer: str
         :param reduce_output: defines how to reduce the output tensor of
                the convolutional layers along the `s` sequence length
@@ -534,9 +542,10 @@ class ParallelCNN(Encoder):
         return self.parallel_conv1d.output_shape
 
 
+@DeveloperAPI
 @register_sequence_encoder("stacked_cnn")
 @register_encoder("stacked_cnn", [AUDIO, SEQUENCE, TEXT, TIMESERIES])
-class StackedCNN(Encoder):
+class StackedCNN(SequenceEncoder):
     def __init__(
         self,
         should_embed=True,
@@ -694,7 +703,7 @@ class StackedCNN(Encoder):
                other keys for its parameters,
                e.g. `{type: normal, mean: 0, stddev: 0}`.
                To know the parameters of each initializer, please refer
-               to TensorFlow's documentation.
+               to PyTorch's documentation.
         :type initializer: str
         :param reduce_output: defines how to reduce the output tensor of
                the convolutional layers along the `s` sequence length
@@ -871,9 +880,10 @@ class StackedCNN(Encoder):
         return {"encoder_output": hidden}
 
 
+@DeveloperAPI
 @register_sequence_encoder("stacked_parallel_cnn")
 @register_encoder("stacked_parallel_cnn", [AUDIO, SEQUENCE, TEXT, TIMESERIES])
-class StackedParallelCNN(Encoder):
+class StackedParallelCNN(SequenceEncoder):
     def __init__(
         self,
         should_embed=True,
@@ -1031,7 +1041,7 @@ class StackedParallelCNN(Encoder):
                other keys for its parameters,
                e.g. `{type: normal, mean: 0, stddev: 0}`.
                To know the parameters of each initializer, please refer
-               to TensorFlow's documentation.
+               to PyTorch's documentation.
         :type initializer: str
         :param reduce_output: defines how to reduce the output tensor of
                the convolutional layers along the `s` sequence length
@@ -1183,9 +1193,10 @@ class StackedParallelCNN(Encoder):
         return {"encoder_output": hidden}
 
 
+@DeveloperAPI
 @register_sequence_encoder("rnn")
 @register_encoder("rnn", [AUDIO, SEQUENCE, TEXT, TIMESERIES])
-class StackedRNN(Encoder):
+class StackedRNN(SequenceEncoder):
     def __init__(
         self,
         should_embed=True,
@@ -1300,12 +1311,9 @@ class StackedRNN(Encoder):
         :param num_rec_layers: the number of stacked recurrent layers.
         :type num_rec_layers: Integer
         :param cell_type: the type of recurrent cell to use.
-               Available values are: `rnn`, `lstm`, `lstm_block`, `lstm`,
-               `ln`, `lstm_cudnn`, `gru`, `gru_block`, `gru_cudnn`.
+               Available values are: `rnn`, `lstm`, `gru`.
                For reference about the differences between the cells please
-               refer to TensorFlow's documentation. We suggest to use the
-               `block` variants on CPU and the `cudnn` variants on GPU
-               because of their increased speed.
+               refer to PyTorch's documentation.
         :type cell_type: str
         :param state_size: the size of the state of the rnn.
         :type state_size: Integer
@@ -1329,7 +1337,7 @@ class StackedRNN(Encoder):
                other keys for its parameters,
                e.g. `{type: normal, mean: 0, stddev: 0}`.
                To know the parameters of each initializer, please refer
-               to TensorFlow's documentation.
+               to PyTorch's documentation.
         :type initializer: str
         :param reduce_output: defines how to reduce the output tensor of
                the convolutional layers along the `s` sequence length
@@ -1456,9 +1464,10 @@ class StackedRNN(Encoder):
         return {"encoder_output": hidden, "encoder_output_state": final_state}
 
 
+@DeveloperAPI
 @register_sequence_encoder("cnnrnn")
 @register_encoder("cnnrnn", [AUDIO, SEQUENCE, TEXT, TIMESERIES])
-class StackedCNNRNN(Encoder):
+class StackedCNNRNN(SequenceEncoder):
     def __init__(
         self,
         should_embed=True,
@@ -1552,12 +1561,9 @@ class StackedCNNRNN(Encoder):
         :param num_layers: the number of stacked recurrent layers.
         :type num_layers: Integer
         :param cell_type: the type of recurrent cell to use.
-               Available values are: `rnn`, `lstm`, `lstm_block`, `lstm`,
-               `ln`, `lstm_cudnn`, `gru`, `gru_block`, `gru_cudnn`.
+               Available values are: `rnn`, `lstm`, `gru`.
                For reference about the differences between the cells please
-               refer to TensorFlow's documentation. We suggest to use the
-               `block` variants on CPU and the `cudnn` variants on GPU
-               because of their increased speed.
+               refer to PyTorch's documentation.
         :type cell_type: str
         :param state_size: the size of the state of the rnn.
         :type state_size: Integer
@@ -1581,7 +1587,7 @@ class StackedCNNRNN(Encoder):
                other keys for its parameters,
                e.g. `{type: normal, mean: 0, stddev: 0}`.
                To know the parameters of each initializer, please refer
-               to TensorFlow's documentation.
+               to PyTorch's documentation.
         :type initializer: str
         :param reduce_output: defines how to reduce the output tensor of
                the convolutional layers along the `s` sequence length
@@ -1745,8 +1751,9 @@ class StackedCNNRNN(Encoder):
         return {"encoder_output": hidden, "encoder_output_state": final_state}
 
 
+@DeveloperAPI
 @register_encoder("transformer", [SEQUENCE, TEXT, TIMESERIES])
-class StackedTransformer(Encoder):
+class StackedTransformer(SequenceEncoder):
     def __init__(
         self,
         max_sequence_length,
@@ -1859,7 +1866,7 @@ class StackedTransformer(Encoder):
                Available values are: `rnn`, `lstm`, `lstm_block`, `lstm`,
                `ln`, `lstm_cudnn`, `gru`, `gru_block`, `gru_cudnn`.
                For reference about the differences between the cells please
-               refer to TensorFlow's documentation. We suggest to use the
+               refer to PyTorch's documentation. We suggest to use the
                `block` variants on CPU and the `cudnn` variants on GPU
                because of their increased speed.
         :type cell_type: str
@@ -1883,7 +1890,7 @@ class StackedTransformer(Encoder):
                other keys for its parameters,
                e.g. `{type: normal, mean: 0, stddev: 0}`.
                To know the parameters of each initializer, please refer
-               to TensorFlow's documentation.
+               to PyTorch's documentation.
         :type initializer: str
         :param reduce_output: defines how to reduce the output tensor of
                the convolutional layers along the `s` sequence length
