@@ -16,51 +16,42 @@
 
 import logging
 from functools import partial
-from typing import Any, Dict, List, Union
+from typing import Dict, List, Union
 
 import numpy as np
 import torch
 
 from ludwig.constants import (
     COLUMN,
-    DECODER,
-    DEPENDENCIES,
-    EDIT_DISTANCE,
-    ENCODER,
-    LAST_ACCURACY,
     LAST_PREDICTIONS,
     LENGTHS,
-    LOSS,
     NAME,
-    PERPLEXITY,
     PREDICTIONS,
     PROBABILITIES,
     PROBABILITY,
     PROC_COLUMN,
-    REDUCE_DEPENDENCIES,
-    REDUCE_INPUT,
     SEQUENCE,
-    SEQUENCE_ACCURACY,
-    TIED,
-    TOKEN_ACCURACY,
-    TYPE,
 )
 from ludwig.features.base_feature import BaseFeatureMixin, InputFeature, OutputFeature, PredictModule
 from ludwig.features.feature_utils import compute_sequence_probability, compute_token_probabilities
 from ludwig.schema.features.sequence_feature import SequenceInputFeatureConfig, SequenceOutputFeatureConfig
-from ludwig.schema.features.utils import register_input_feature, register_output_feature
+from ludwig.types import (
+    FeatureMetadataDict,
+    FeaturePostProcessingOutputDict,
+    PreprocessingConfigDict,
+    TrainingSetMetadataDict,
+)
 from ludwig.utils import output_feature_utils
 from ludwig.utils.math_utils import softmax
-from ludwig.utils.misc_utils import get_from_registry, set_default_value, set_default_values
 from ludwig.utils.strings_utils import (
     build_sequence_matrix,
     create_vocabulary,
     SpecialSymbol,
     START_SYMBOL,
     STOP_SYMBOL,
-    tokenizer_registry,
     UNKNOWN_SYMBOL,
 )
+from ludwig.utils.tokenizers import get_tokenizer_from_registry
 from ludwig.utils.types import DataFrame, TorchscriptPreprocessingInput
 
 logger = logging.getLogger(__name__)
@@ -69,11 +60,11 @@ logger = logging.getLogger(__name__)
 class _SequencePreprocessing(torch.nn.Module):
     """Torchscript-enabled version of preprocessing done by SequenceFeatureMixin.add_feature_data."""
 
-    def __init__(self, metadata: Dict[str, Any]):
+    def __init__(self, metadata: TrainingSetMetadataDict):
         super().__init__()
         self.lowercase = metadata["preprocessing"]["lowercase"]
         self.tokenizer_type = metadata["preprocessing"]["tokenizer"]
-        self.tokenizer = get_from_registry(self.tokenizer_type, tokenizer_registry)(
+        self.tokenizer = get_tokenizer_from_registry(self.tokenizer_type)(
             pretrained_model_name_or_path=metadata["preprocessing"].get("pretrained_model_name_or_path", None)
         )
 
@@ -120,10 +111,12 @@ class _SequencePreprocessing(torch.nn.Module):
         sequence_vector = torch.full([self.max_sequence_length], self.unit_to_id[self.padding_symbol])
 
         if self.tokenizer_type == "hf_tokenizer":
+            # Handles start, stop, and unknown symbols implicitly
             unit_sequence = self.tokenizer(sequence)
             assert torch.jit.isinstance(unit_sequence, List[int])
-            # Handles start, stop, and unknown symbols implicitly
-            sequence_vector[: len(unit_sequence)] = torch.tensor(unit_sequence)
+            # Ensures that the sequence lengths are aligned between the input and output tensors.
+            sequence_length = min(len(unit_sequence), self.max_sequence_length)
+            sequence_vector[:sequence_length] = torch.tensor(unit_sequence)[:sequence_length]
             return sequence_vector
 
         # If tokenizer is not HF, we manually convert tokens to IDs and insert start, stop, and unknown symbols.
@@ -148,7 +141,7 @@ class _SequencePreprocessing(torch.nn.Module):
 
 
 class _SequencePostprocessing(torch.nn.Module):
-    def __init__(self, metadata: Dict[str, Any]):
+    def __init__(self, metadata: TrainingSetMetadataDict):
         super().__init__()
         self.max_sequence_length = int(metadata["max_sequence_length"])
         self.idx2str = metadata["idx2str"]
@@ -157,7 +150,7 @@ class _SequencePostprocessing(torch.nn.Module):
         self.probabilities_key = PROBABILITIES
         self.probability_key = PROBABILITY
 
-    def forward(self, preds: Dict[str, torch.Tensor], feature_name: str) -> Dict[str, Any]:
+    def forward(self, preds: Dict[str, torch.Tensor], feature_name: str) -> FeaturePostProcessingOutputDict:
         pred_predictions = output_feature_utils.get_output_feature_tensor(preds, feature_name, self.predictions_key)
         pred_probabilities = output_feature_utils.get_output_feature_tensor(preds, feature_name, self.probabilities_key)
 
@@ -200,16 +193,14 @@ class SequenceFeatureMixin(BaseFeatureMixin):
     def type():
         return SEQUENCE
 
-    @staticmethod
-    def preprocessing_defaults():
-        return SequenceInputFeatureConfig().preprocessing.to_dict()
-
     @staticmethod
     def cast_column(column, backend):
         return column.astype(str)
 
     @staticmethod
-    def get_feature_meta(column, preprocessing_parameters, backend):
+    def get_feature_meta(
+        column, preprocessing_parameters: PreprocessingConfigDict, backend, is_input_feature: bool
+    ) -> FeatureMetadataDict:
         idx2str, str2idx, str2freq, max_length, _, _, _, _ = create_vocabulary(
             column,
             preprocessing_parameters["tokenizer"],
@@ -218,6 +209,7 @@ class SequenceFeatureMixin(BaseFeatureMixin):
             vocab_file=preprocessing_parameters["vocab_file"],
             unknown_symbol=preprocessing_parameters["unknown_symbol"],
             padding_symbol=preprocessing_parameters["padding_symbol"],
+            ngram_size=preprocessing_parameters["ngram_size"],
             processor=backend.df_engine,
         )
         max_length = min(preprocessing_parameters["max_sequence_length"], max_length)
@@ -230,7 +222,7 @@ class SequenceFeatureMixin(BaseFeatureMixin):
         }
 
     @staticmethod
-    def feature_data(column, metadata, preprocessing_parameters, backend):
+    def feature_data(column, metadata, preprocessing_parameters: PreprocessingConfigDict, backend):
         sequence_data = build_sequence_matrix(
             sequences=column,
             inverse_vocabulary=metadata["str2idx"],
@@ -247,7 +239,13 @@ class SequenceFeatureMixin(BaseFeatureMixin):
 
     @staticmethod
     def add_feature_data(
-        feature_config, input_df, proc_df, metadata, preprocessing_parameters, backend, skip_save_processed_input
+        feature_config,
+        input_df,
+        proc_df,
+        metadata,
+        preprocessing_parameters: PreprocessingConfigDict,
+        backend,
+        skip_save_processed_input,
     ):
         sequence_data = SequenceInputFeature.feature_data(
             input_df[feature_config[COLUMN]],
@@ -259,10 +257,8 @@ class SequenceFeatureMixin(BaseFeatureMixin):
         return proc_df
 
 
-@register_input_feature(SEQUENCE)
 class SequenceInputFeature(SequenceFeatureMixin, InputFeature):
-    def __init__(self, input_feature_config: Union[SequenceInputFeatureConfig, Dict], encoder_obj=None, **kwargs):
-        input_feature_config = self.load_config(input_feature_config)
+    def __init__(self, input_feature_config: SequenceInputFeatureConfig, encoder_obj=None, **kwargs):
         super().__init__(input_feature_config, **kwargs)
 
         if encoder_obj:
@@ -286,16 +282,10 @@ class SequenceInputFeature(SequenceFeatureMixin, InputFeature):
         return torch.int32
 
     @staticmethod
-    def update_config_with_metadata(input_feature, feature_metadata, *args, **kwargs):
-        input_feature[ENCODER]["vocab"] = feature_metadata["idx2str"]
-        input_feature[ENCODER]["vocab_size"] = len(feature_metadata["idx2str"])
-        input_feature[ENCODER]["max_sequence_length"] = feature_metadata["max_sequence_length"]
-
-    @staticmethod
-    def populate_defaults(input_feature):
-        defaults = SequenceInputFeatureConfig()
-        set_default_value(input_feature, TIED, defaults.tied)
-        set_default_values(input_feature, {ENCODER: {TYPE: defaults.encoder.type}})
+    def update_config_with_metadata(feature_config, feature_metadata, *args, **kwargs):
+        feature_config.encoder.vocab = feature_metadata["idx2str"]
+        feature_config.encoder.vocab_size = len(feature_metadata["idx2str"])
+        feature_config.encoder.max_sequence_length = feature_metadata["max_sequence_length"]
 
     @staticmethod
     def get_schema_cls():
@@ -310,29 +300,17 @@ class SequenceInputFeature(SequenceFeatureMixin, InputFeature):
         return self.encoder_obj.output_shape
 
     @staticmethod
-    def create_preproc_module(metadata: Dict[str, Any]) -> torch.nn.Module:
+    def create_preproc_module(metadata: TrainingSetMetadataDict) -> torch.nn.Module:
         return _SequencePreprocessing(metadata)
 
 
-@register_output_feature(SEQUENCE)
 class SequenceOutputFeature(SequenceFeatureMixin, OutputFeature):
-    metric_functions = {
-        LOSS: None,
-        TOKEN_ACCURACY: None,
-        SEQUENCE_ACCURACY: None,
-        LAST_ACCURACY: None,
-        PERPLEXITY: None,
-        EDIT_DISTANCE: None,
-    }
-    default_validation_metric = LOSS
-
     def __init__(
         self,
         output_feature_config: Union[SequenceOutputFeatureConfig, Dict],
         output_features: Dict[str, OutputFeature],
         **kwargs,
     ):
-        output_feature_config = self.load_config(output_feature_config)
         super().__init__(output_feature_config, output_features, **kwargs)
         self.decoder_obj = self.initialize_decoder(output_feature_config.decoder)
         self._setup_loss()
@@ -361,27 +339,27 @@ class SequenceOutputFeature(SequenceFeatureMixin, OutputFeature):
         return torch.Size([self.decoder_obj.config.max_sequence_length])
 
     @staticmethod
-    def update_config_with_metadata(output_feature, feature_metadata, *args, **kwargs):
-        output_feature[DECODER]["vocab_size"] = feature_metadata["vocab_size"]
-        output_feature[DECODER]["max_sequence_length"] = feature_metadata["max_sequence_length"]
-        if isinstance(output_feature[LOSS]["class_weights"], (list, tuple)):
-            if len(output_feature[LOSS]["class_weights"]) != output_feature[DECODER]["vocab_size"]:
+    def update_config_with_metadata(feature_config, feature_metadata, *args, **kwargs):
+        feature_config.decoder.vocab_size = feature_metadata["vocab_size"]
+        feature_config.decoder.max_sequence_length = feature_metadata["max_sequence_length"]
+        if isinstance(feature_config.loss.class_weights, (list, tuple)):
+            if len(feature_config.loss.class_weights) != feature_config.decoder.vocab_size:
                 raise ValueError(
                     "The length of class_weights ({}) is not compatible with "
                     "the number of classes ({}) for feature {}. "
                     "Check the metadata JSON file to see the classes "
                     "and their order and consider there needs to be a weight "
                     "for the <UNK> and <PAD> class too.".format(
-                        len(output_feature[LOSS]["class_weights"]),
-                        output_feature[DECODER]["vocab_size"],
-                        output_feature[COLUMN],
+                        len(feature_config.loss.class_weights),
+                        feature_config.decoder.vocab_size,
+                        feature_config.column,
                     )
                 )
 
-        if output_feature[LOSS]["class_similarities_temperature"] > 0:
-            if "class_similarities" in output_feature[LOSS]:
-                similarities = output_feature[LOSS]["class_similarities"]
-                temperature = output_feature[LOSS]["class_similarities_temperature"]
+        if feature_config.loss.class_similarities_temperature > 0:
+            if "class_similarities" in feature_config.loss:
+                similarities = feature_config.loss.class_similarities
+                temperature = feature_config.loss.class_similarities_temperature
 
                 curr_row = 0
                 first_row_length = 0
@@ -399,7 +377,7 @@ class SequenceOutputFeature(SequenceFeatureMixin, OutputFeature):
                                 "of {} is {}, different from the length of "
                                 "the first row {}. All rows must have "
                                 "the same length.".format(
-                                    curr_row, output_feature[COLUMN], curr_row_length, first_row_length
+                                    curr_row, feature_config.column, curr_row_length, first_row_length
                                 )
                             )
                         else:
@@ -411,30 +389,30 @@ class SequenceOutputFeature(SequenceFeatureMixin, OutputFeature):
                         "The class_similarities matrix of {} has "
                         "{} rows and {} columns, "
                         "their number must be identical.".format(
-                            output_feature[COLUMN], len(similarities), all_rows_length
+                            feature_config.column, len(similarities), all_rows_length
                         )
                     )
 
-                if all_rows_length != output_feature[DECODER]["vocab_size"]:
+                if all_rows_length != feature_config.decoder.vocab_size:
                     raise ValueError(
                         "The size of the class_similarities matrix of {} is "
                         "{}, different from the number of classes ({}). "
                         "Check the metadata JSON file to see the classes "
                         "and their order and "
                         "consider <UNK> and <PAD> class too.".format(
-                            output_feature[COLUMN], all_rows_length, output_feature[DECODER]["vocab_size"]
+                            feature_config.column, all_rows_length, feature_config.decoder.vocab_size
                         )
                     )
 
                 similarities = np.array(similarities, dtype=np.float32)
                 for i in range(len(similarities)):
                     similarities[i, :] = softmax(similarities[i, :], temperature=temperature)
-                output_feature[LOSS]["class_similarities"] = similarities
+                feature_config.loss.class_similarities = similarities
             else:
                 raise ValueError(
                     "class_similarities_temperature > 0, "
                     "but no class_similarities are provided "
-                    "for feature {}".format(output_feature[COLUMN])
+                    "for feature {}".format(feature_config.column)
                 )
 
     @staticmethod
@@ -495,28 +473,7 @@ class SequenceOutputFeature(SequenceFeatureMixin, OutputFeature):
         return result
 
     @staticmethod
-    def populate_defaults(output_feature):
-        defaults = SequenceOutputFeatureConfig()
-        set_default_value(output_feature, LOSS, {})
-        set_default_values(output_feature[LOSS], defaults.loss.Schema().dump(defaults.loss))
-
-        if DECODER in output_feature and TYPE in output_feature[DECODER] and output_feature[DECODER][TYPE] == "tagger":
-            set_default_value(output_feature, "reduce_input", None)
-
-        set_default_values(
-            output_feature,
-            {
-                DECODER: {
-                    TYPE: defaults.decoder.type,
-                },
-                DEPENDENCIES: defaults.dependencies,
-                REDUCE_INPUT: defaults.reduce_input,
-                REDUCE_DEPENDENCIES: defaults.reduce_dependencies,
-            },
-        )
-
-    @staticmethod
-    def create_postproc_module(metadata: Dict[str, Any]) -> torch.nn.Module:
+    def create_postproc_module(metadata: TrainingSetMetadataDict) -> torch.nn.Module:
         return _SequencePostprocessing(metadata)
 
     @staticmethod
