@@ -15,31 +15,22 @@
 # ==============================================================================
 import logging
 from functools import partial
-from typing import Any, Dict, Union
+from typing import Dict, Union
 
 import torch
 
 from ludwig.constants import (
     COLUMN,
-    DECODER,
-    EDIT_DISTANCE,
-    ENCODER,
-    LAST_ACCURACY,
     LAST_PREDICTIONS,
     LENGTHS,
-    LOSS,
     NAME,
-    PERPLEXITY,
     PREDICTIONS,
+    PREPROCESSING,
     PROBABILITIES,
     PROBABILITY,
     PROC_COLUMN,
     TEXT,
-    TIED,
-    TOKEN_ACCURACY,
-    TYPE,
 )
-from ludwig.encoders.registry import get_encoder_cls
 from ludwig.features.base_feature import BaseFeatureMixin, OutputFeature
 from ludwig.features.feature_utils import compute_sequence_probability, compute_token_probabilities
 from ludwig.features.sequence_feature import (
@@ -49,9 +40,8 @@ from ludwig.features.sequence_feature import (
     SequenceOutputFeature,
 )
 from ludwig.schema.features.text_feature import TextInputFeatureConfig, TextOutputFeatureConfig
-from ludwig.schema.features.utils import register_input_feature, register_output_feature
+from ludwig.types import FeatureMetadataDict, PreprocessingConfigDict, TrainingSetMetadataDict
 from ludwig.utils.math_utils import softmax
-from ludwig.utils.misc_utils import set_default_value, set_default_values
 from ludwig.utils.strings_utils import build_sequence_matrix, create_vocabulary, SpecialSymbol, UNKNOWN_SYMBOL
 from ludwig.utils.types import DataFrame
 
@@ -63,16 +53,12 @@ class TextFeatureMixin(BaseFeatureMixin):
     def type():
         return TEXT
 
-    @staticmethod
-    def preprocessing_defaults():
-        return TextInputFeatureConfig().preprocessing.to_dict()
-
     @staticmethod
     def cast_column(column, backend):
         return column.astype(str)
 
     @staticmethod
-    def feature_meta(column, preprocessing_parameters, backend):
+    def feature_meta(column, preprocessing_parameters: PreprocessingConfigDict, backend):
         (
             idx2str,
             str2idx,
@@ -91,6 +77,7 @@ class TextFeatureMixin(BaseFeatureMixin):
             unknown_symbol=preprocessing_parameters["unknown_symbol"],
             padding_symbol=preprocessing_parameters["padding_symbol"],
             pretrained_model_name_or_path=preprocessing_parameters["pretrained_model_name_or_path"],
+            ngram_size=preprocessing_parameters["ngram_size"],
             processor=backend.df_engine,
         )
         return (
@@ -105,7 +92,9 @@ class TextFeatureMixin(BaseFeatureMixin):
         )
 
     @staticmethod
-    def get_feature_meta(column, preprocessing_parameters, backend):
+    def get_feature_meta(
+        column, preprocessing_parameters: PreprocessingConfigDict, backend, is_input_feature: bool
+    ) -> FeatureMetadataDict:
         tf_meta = TextFeatureMixin.feature_meta(column, preprocessing_parameters, backend)
         (
             idx2str,
@@ -132,7 +121,7 @@ class TextFeatureMixin(BaseFeatureMixin):
         }
 
     @staticmethod
-    def feature_data(column, metadata, preprocessing_parameters, backend):
+    def feature_data(column, metadata, preprocessing_parameters: PreprocessingConfigDict, backend):
         # TODO(1891): Remove backward compatibility hack once all models have been retrained with Ludwig after
         # https://github.com/ludwig-ai/ludwig/pull/1859.
         prefix = ""
@@ -170,7 +159,13 @@ class TextFeatureMixin(BaseFeatureMixin):
 
     @staticmethod
     def add_feature_data(
-        feature_config, input_df, proc_df, metadata, preprocessing_parameters, backend, skip_save_processed_input
+        feature_config,
+        input_df,
+        proc_df,
+        metadata,
+        preprocessing_parameters: PreprocessingConfigDict,
+        backend,
+        skip_save_processed_input,
     ):
         proc_df[feature_config[PROC_COLUMN]] = TextFeatureMixin.feature_data(
             input_df[feature_config[COLUMN]],
@@ -181,10 +176,8 @@ class TextFeatureMixin(BaseFeatureMixin):
         return proc_df
 
 
-@register_input_feature(TEXT)
 class TextInputFeature(TextFeatureMixin, SequenceInputFeature):
-    def __init__(self, input_feature_config: Union[TextInputFeatureConfig, Dict], encoder_obj=None, **kwargs):
-        input_feature_config = self.load_config(input_feature_config)
+    def __init__(self, input_feature_config: TextInputFeatureConfig, encoder_obj=None, **kwargs):
         super().__init__(input_feature_config, encoder_obj=encoder_obj, **kwargs)
 
     def forward(self, inputs, mask=None):
@@ -214,24 +207,17 @@ class TextInputFeature(TextFeatureMixin, SequenceInputFeature):
     def input_shape(self):
         return torch.Size([self.encoder_obj.config.max_sequence_length])
 
-    @staticmethod
-    def update_config_with_metadata(input_feature, feature_metadata, *args, **kwargs):
-        input_feature[ENCODER]["vocab"] = feature_metadata["idx2str"]
-        input_feature[ENCODER]["vocab_size"] = len(feature_metadata["idx2str"])
-        input_feature[ENCODER]["max_sequence_length"] = feature_metadata["max_sequence_length"]
-        input_feature[ENCODER]["pad_idx"] = feature_metadata["pad_idx"]
-        input_feature[ENCODER]["num_tokens"] = len(feature_metadata["idx2str"])
+    def update_config_after_module_init(self, feature_config):
+        feature_config.encoder = self.encoder_obj.config
 
     @staticmethod
-    def populate_defaults(input_feature):
-        defaults = TextInputFeatureConfig()
-        set_default_value(input_feature, TIED, defaults.tied)
-        set_default_values(input_feature, {ENCODER: {TYPE: defaults.encoder.type}})
-
-        encoder_class = get_encoder_cls(input_feature[TYPE], input_feature[ENCODER][TYPE])
-
-        if hasattr(encoder_class, "default_params"):
-            set_default_values(input_feature, encoder_class.default_params)
+    def update_config_with_metadata(feature_config, feature_metadata, *args, **kwargs):
+        feature_config.encoder.vocab = feature_metadata["idx2str"]
+        feature_config.encoder.vocab_size = len(feature_metadata["idx2str"])
+        feature_config.encoder.max_sequence_length = feature_metadata["max_sequence_length"]
+        feature_config.encoder.pad_idx = feature_metadata["pad_idx"]
+        feature_config.encoder.num_tokens = len(feature_metadata["idx2str"])
+        feature_config.encoder.skip = feature_metadata[PREPROCESSING].get("cache_encoder_embeddings", False)
 
     @staticmethod
     def get_schema_cls():
@@ -242,22 +228,17 @@ class TextInputFeature(TextFeatureMixin, SequenceInputFeature):
         return self.encoder_obj.output_shape
 
     @staticmethod
-    def create_preproc_module(metadata: Dict[str, Any]) -> torch.nn.Module:
+    def create_preproc_module(metadata: TrainingSetMetadataDict) -> torch.nn.Module:
         return _SequencePreprocessing(metadata)
 
 
-@register_output_feature(TEXT)
 class TextOutputFeature(TextFeatureMixin, SequenceOutputFeature):
-    metric_functions = {LOSS: None, TOKEN_ACCURACY: None, LAST_ACCURACY: None, PERPLEXITY: None, EDIT_DISTANCE: None}
-    default_validation_metric = LOSS
-
     def __init__(
         self,
-        output_feature_config: Union[TextInputFeatureConfig, Dict],
+        output_feature_config: Union[TextOutputFeatureConfig, Dict],
         output_features: Dict[str, OutputFeature],
         **kwargs,
     ):
-        output_feature_config = self.load_config(output_feature_config)
         super().__init__(output_feature_config, output_features, **kwargs)
 
     @classmethod
@@ -269,32 +250,32 @@ class TextOutputFeature(TextFeatureMixin, SequenceOutputFeature):
         return torch.Size([self.decoder_obj.config.max_sequence_length])
 
     @staticmethod
-    def update_config_with_metadata(output_feature, feature_metadata, *args, **kwargs):
-        output_feature[DECODER]["vocab_size"] = feature_metadata["vocab_size"]
-        output_feature[DECODER]["max_sequence_length"] = feature_metadata["max_sequence_length"]
-        if isinstance(output_feature[LOSS]["class_weights"], (list, tuple)):
+    def update_config_with_metadata(feature_config, feature_metadata, *args, **kwargs):
+        feature_config.decoder.vocab_size = feature_metadata["vocab_size"]
+        feature_config.decoder.max_sequence_length = feature_metadata["max_sequence_length"]
+        if isinstance(feature_config.loss.class_weights, (list, tuple)):
             # [0, 0] for UNK and PAD
-            output_feature[LOSS]["class_weights"] = [0, 0] + output_feature[LOSS]["class_weights"]
-            if len(output_feature[LOSS]["class_weights"]) != output_feature[DECODER]["vocab_size"]:
+            feature_config.loss.class_weights = [0, 0] + feature_config.loss.class_weights
+            if len(feature_config.loss.class_weights) != feature_config.decoder.vocab_size:
                 raise ValueError(
                     "The length of class_weights ({}) is not compatible with "
                     "the number of classes ({})".format(
-                        len(output_feature[LOSS]["class_weights"]), output_feature[DECODER]["vocab_size"]
+                        len(feature_config.loss.class_weights), feature_config.decoder.vocab_size
                     )
                 )
 
-        if output_feature[LOSS]["class_similarities_temperature"] > 0:
-            if "class_similarities" in output_feature:
-                distances = output_feature["class_similarities"]
-                temperature = output_feature[LOSS]["class_similarities_temperature"]
+        if feature_config.loss.class_similarities_temperature > 0:
+            if feature_config.class_similarities:
+                distances = feature_config.class_similarities
+                temperature = feature_config.loss.class_similarities_temperature
                 for i in range(len(distances)):
                     distances[i, :] = softmax(distances[i, :], temperature=temperature)
-                output_feature[LOSS]["class_similarities"] = distances
+                feature_config.loss.class_similarities = distances
             else:
                 raise ValueError(
                     "class_similarities_temperature > 0,"
                     "but no class similarities are provided "
-                    "for feature {}".format(output_feature[COLUMN])
+                    "for feature {}".format(feature_config.column)
                 )
 
     @staticmethod
@@ -353,13 +334,9 @@ class TextOutputFeature(TextFeatureMixin, SequenceOutputFeature):
         return result
 
     @staticmethod
-    def create_postproc_module(metadata: Dict[str, Any]) -> torch.nn.Module:
+    def create_postproc_module(metadata: TrainingSetMetadataDict) -> torch.nn.Module:
         return _SequencePostprocessing(metadata)
 
-    @staticmethod
-    def populate_defaults(output_feature):
-        SequenceOutputFeature.populate_defaults(output_feature)
-
     @staticmethod
     def get_schema_cls():
         return TextOutputFeatureConfig
