@@ -14,19 +14,17 @@
 # limitations under the License.
 # ==============================================================================
 import logging
-from typing import Any, Dict, List, Union
+from typing import List
 
 import numpy as np
 import torch
 
-from ludwig.constants import COLUMN, ENCODER, NAME, PROC_COLUMN, TIED, TIMESERIES, TYPE
+from ludwig.constants import COLUMN, NAME, PROC_COLUMN, TIMESERIES
 from ludwig.features.base_feature import BaseFeatureMixin
 from ludwig.features.sequence_feature import SequenceInputFeature
 from ludwig.schema.features.timeseries_feature import TimeseriesInputFeatureConfig
-from ludwig.schema.features.utils import register_input_feature
-from ludwig.utils.misc_utils import get_from_registry, set_default_value, set_default_values
-from ludwig.utils.strings_utils import tokenizer_registry
-from ludwig.utils.tokenizers import TORCHSCRIPT_COMPATIBLE_TOKENIZERS
+from ludwig.types import FeatureMetadataDict, PreprocessingConfigDict, TrainingSetMetadataDict
+from ludwig.utils.tokenizers import get_tokenizer_from_registry, TORCHSCRIPT_COMPATIBLE_TOKENIZERS
 from ludwig.utils.types import TorchscriptPreprocessingInput
 
 logger = logging.getLogger(__name__)
@@ -35,14 +33,14 @@ logger = logging.getLogger(__name__)
 class _TimeseriesPreprocessing(torch.nn.Module):
     """Torchscript-enabled version of preprocessing done by TimeseriesFeatureMixin.add_feature_data."""
 
-    def __init__(self, metadata: Dict[str, Any]):
+    def __init__(self, metadata: TrainingSetMetadataDict):
         super().__init__()
         if metadata["preprocessing"]["tokenizer"] not in TORCHSCRIPT_COMPATIBLE_TOKENIZERS:
             raise ValueError(
                 f"{metadata['preprocessing']['tokenizer']} is not supported by torchscript. Please use "
                 f"one of {TORCHSCRIPT_COMPATIBLE_TOKENIZERS}."
             )
-        self.tokenizer = get_from_registry(metadata["preprocessing"]["tokenizer"], tokenizer_registry)()
+        self.tokenizer = get_tokenizer_from_registry(metadata["preprocessing"]["tokenizer"])()
         self.padding = metadata["preprocessing"]["padding"]
         self.padding_value = float(metadata["preprocessing"]["padding_value"])
         self.max_timeseries_length = int(metadata["max_timeseries_length"])
@@ -107,18 +105,16 @@ class TimeseriesFeatureMixin(BaseFeatureMixin):
     def type():
         return TIMESERIES
 
-    @staticmethod
-    def preprocessing_defaults():
-        return TimeseriesInputFeatureConfig().preprocessing.to_dict()
-
     @staticmethod
     def cast_column(column, backend):
         return column
 
     @staticmethod
-    def get_feature_meta(column, preprocessing_parameters, backend):
+    def get_feature_meta(
+        column, preprocessing_parameters: PreprocessingConfigDict, backend, is_input_feature: bool
+    ) -> FeatureMetadataDict:
         column = column.astype(str)
-        tokenizer = get_from_registry(preprocessing_parameters["tokenizer"], tokenizer_registry)()
+        tokenizer = get_tokenizer_from_registry(preprocessing_parameters["tokenizer"])()
         max_length = 0
         for timeseries in column:
             processed_line = tokenizer(timeseries)
@@ -129,7 +125,7 @@ class TimeseriesFeatureMixin(BaseFeatureMixin):
 
     @staticmethod
     def build_matrix(timeseries, tokenizer_name, length_limit, padding_value, padding, backend):
-        tokenizer = get_from_registry(tokenizer_name, tokenizer_registry)()
+        tokenizer = get_tokenizer_from_registry(tokenizer_name)()
 
         ts_vectors = backend.df_engine.map_objects(timeseries, lambda ts: np.array(tokenizer(ts)).astype(np.float32))
 
@@ -150,7 +146,7 @@ class TimeseriesFeatureMixin(BaseFeatureMixin):
         return backend.df_engine.map_objects(ts_vectors, pad)
 
     @staticmethod
-    def feature_data(column, metadata, preprocessing_parameters, backend):
+    def feature_data(column, metadata, preprocessing_parameters: PreprocessingConfigDict, backend):
         timeseries_data = TimeseriesFeatureMixin.build_matrix(
             column,
             preprocessing_parameters["tokenizer"],
@@ -163,7 +159,13 @@ class TimeseriesFeatureMixin(BaseFeatureMixin):
 
     @staticmethod
     def add_feature_data(
-        feature_config, input_df, proc_df, metadata, preprocessing_parameters, backend, skip_save_processed_input
+        feature_config,
+        input_df,
+        proc_df,
+        metadata,
+        preprocessing_parameters: PreprocessingConfigDict,
+        backend,
+        skip_save_processed_input,
     ):
         proc_df[feature_config[PROC_COLUMN]] = TimeseriesFeatureMixin.feature_data(
             input_df[feature_config[COLUMN]].astype(str),
@@ -174,10 +176,8 @@ class TimeseriesFeatureMixin(BaseFeatureMixin):
         return proc_df
 
 
-@register_input_feature(TIMESERIES)
 class TimeseriesInputFeature(TimeseriesFeatureMixin, SequenceInputFeature):
-    def __init__(self, input_feature_config: Union[TimeseriesInputFeatureConfig, Dict], encoder_obj=None, **kwargs):
-        input_feature_config = self.load_config(input_feature_config)
+    def __init__(self, input_feature_config: TimeseriesInputFeatureConfig, encoder_obj=None, **kwargs):
         # add required sequence encoder parameters for time series
         input_feature_config.encoder.embedding_size = 1
         input_feature_config.encoder.should_embed = False
@@ -204,21 +204,15 @@ class TimeseriesInputFeature(TimeseriesFeatureMixin, SequenceInputFeature):
         return torch.float32
 
     @staticmethod
-    def update_config_with_metadata(input_feature, feature_metadata, *args, **kwargs):
-        input_feature[ENCODER]["max_sequence_length"] = feature_metadata["max_timeseries_length"]
-
-    @staticmethod
-    def populate_defaults(input_feature):
-        defaults = TimeseriesInputFeatureConfig()
-        set_default_value(input_feature, TIED, defaults.tied)
-        set_default_values(input_feature, {ENCODER: {TYPE: defaults.encoder.type}})
+    def update_config_with_metadata(feature_config, feature_metadata, *args, **kwargs):
+        feature_config.encoder.max_sequence_length = feature_metadata["max_timeseries_length"]
 
     @staticmethod
     def get_schema_cls():
         return TimeseriesInputFeatureConfig
 
     @staticmethod
-    def create_preproc_module(metadata: Dict[str, Any]) -> torch.nn.Module:
+    def create_preproc_module(metadata: TrainingSetMetadataDict) -> torch.nn.Module:
         return _TimeseriesPreprocessing(metadata)
 
 
