@@ -21,7 +21,6 @@ from ludwig.constants import (
     MAX_CONCURRENT_TRIALS,
     METRIC,
     NAME,
-    NUM_SAMPLES,
     OUTPUT_FEATURES,
     PARAMETERS,
     PREPROCESSING,
@@ -33,7 +32,6 @@ from ludwig.constants import (
     VALIDATION,
 )
 from ludwig.data.split import get_splitter
-from ludwig.features.feature_registries import output_type_registry
 from ludwig.hyperopt.results import HyperoptResults
 from ludwig.hyperopt.utils import (
     log_warning_if_all_grid_type_parameters,
@@ -42,15 +40,18 @@ from ludwig.hyperopt.utils import (
     should_tune_preprocessing,
     update_hyperopt_params_with_defaults,
 )
-from ludwig.utils.backward_compatibility import upgrade_to_latest_version
+from ludwig.schema.model_config import ModelConfig
+from ludwig.utils.backward_compatibility import upgrade_config_dict_to_latest_version
 from ludwig.utils.dataset_utils import generate_dataset_statistics
-from ludwig.utils.defaults import default_random_seed, merge_with_defaults
+from ludwig.utils.defaults import default_random_seed
 from ludwig.utils.fs_utils import makedirs, open_file
-from ludwig.utils.misc_utils import get_from_registry
 
 try:
+    from ray.tune import Callback as TuneCallback
+
     from ludwig.backend.ray import RayBackend
 except ImportError:
+    TuneCallback = object
 
     class RayBackend:
         pass
@@ -85,6 +86,7 @@ def hyperopt(
     gpu_memory_limit: Optional[float] = None,
     allow_parallel_threads: bool = True,
     callbacks: List[Callback] = None,
+    tune_callbacks: List[TuneCallback] = None,
     backend: Union[Backend, str] = None,
     random_seed: int = default_random_seed,
     hyperopt_log_verbosity: int = 3,
@@ -175,7 +177,7 @@ def hyperopt(
         for training.
     :param gpu_memory_limit: (float: default: `None`) maximum memory fraction
         [0, 1] allowed to allocate per GPU device.
-    :param allow_parallel_threads: (bool, default: `True`) allow TensorFlow
+    :param allow_parallel_threads: (bool, default: `True`) allow PyTorch
         to use multithreading parallelism to improve performance at
         the cost of determinism.
     :param callbacks: (list, default: `None`) a list of
@@ -203,19 +205,22 @@ def hyperopt(
     else:
         config_dict = config
 
-    # backwards compatibility
-    config = upgrade_to_latest_version(config_dict)
-
-    # Retain pre-merged config for hyperopt schema generation
-    premerged_config = copy.deepcopy(config)
-
-    # merge config with defaults
-    config = merge_with_defaults(config)
-
     if HYPEROPT not in config:
         raise ValueError("Hyperopt Section not present in config")
 
-    hyperopt_config = config[HYPEROPT]
+    # backwards compatibility
+    upgraded_config = upgrade_config_dict_to_latest_version(config_dict)
+
+    # Initialize config object
+    config_obj = ModelConfig.from_dict(upgraded_config)
+
+    # Retain pre-merged config for hyperopt schema generation
+    premerged_config = copy.deepcopy(upgraded_config)
+
+    # Get full config with defaults
+    full_config = config_obj.to_dict()  # TODO (Connor): Refactor to use config object
+
+    hyperopt_config = full_config[HYPEROPT]
 
     # Explicitly default to a local backend to avoid picking up Ray or Horovod
     # backend from the environment.
@@ -225,12 +230,12 @@ def hyperopt(
     update_hyperopt_params_with_defaults(hyperopt_config)
 
     # Check if all features are grid type parameters and log UserWarning if needed
-    log_warning_if_all_grid_type_parameters(hyperopt_config[PARAMETERS], hyperopt_config[EXECUTOR].get(NUM_SAMPLES))
+    log_warning_if_all_grid_type_parameters(hyperopt_config)
 
     # Infer max concurrent trials
     if hyperopt_config[EXECUTOR].get(MAX_CONCURRENT_TRIALS) == AUTO:
         hyperopt_config[EXECUTOR][MAX_CONCURRENT_TRIALS] = backend.max_concurrent_trials(hyperopt_config)
-        logger.info(f"Set max_concurrent_trials to {hyperopt_config[EXECUTOR][MAX_CONCURRENT_TRIALS]}")
+        logger.info(f"Setting max_concurrent_trials to {hyperopt_config[EXECUTOR][MAX_CONCURRENT_TRIALS]}")
 
     # Print hyperopt config
     logger.info("Hyperopt Config")
@@ -248,7 +253,7 @@ def hyperopt(
     ######################
     # check validity of output_feature / metric/ split combination
     ######################
-    splitter = get_splitter(**config[PREPROCESSING]["split"])
+    splitter = get_splitter(**full_config[PREPROCESSING]["split"])
     if split == TRAINING:
         if training_set is None and not splitter.has_split(0):
             raise ValueError(
@@ -281,7 +286,7 @@ def hyperopt(
         if metric != LOSS:
             raise ValueError('The only valid metric for "combined" output feature is "loss"')
     else:
-        output_feature_names = {of[NAME] for of in config[OUTPUT_FEATURES]}
+        output_feature_names = {of[NAME] for of in full_config[OUTPUT_FEATURES]}
         if output_feature not in output_feature_names:
             raise ValueError(
                 'The output feature specified for hyperopt "{}" '
@@ -289,23 +294,6 @@ def hyperopt(
                 'Available ones are: {} and "combined"'.format(output_feature, output_feature_names)
             )
 
-        output_feature_type = None
-        for of in config[OUTPUT_FEATURES]:
-            if of[NAME] == output_feature:
-                output_feature_type = of[TYPE]
-        feature_class = get_from_registry(output_feature_type, output_type_registry)
-        if metric not in feature_class.metric_functions:
-            # todo v0.4: allow users to specify also metrics from the overall
-            #  and per class metrics from the training stats and in general
-            #  and post-processed metric
-            raise ValueError(
-                'The specified metric for hyperopt "{}" is not a valid metric '
-                'for the specified output feature "{}" of type "{}". '
-                "Available metrics are: {}".format(
-                    metric, output_feature, output_feature_type, feature_class.metric_functions.keys()
-                )
-            )
-
     hyperopt_executor = get_build_hyperopt_executor(executor[TYPE])(
         parameters, output_feature, metric, goal, split, search_alg=search_alg, **executor
     )
@@ -321,13 +309,13 @@ def hyperopt(
     for callback in callbacks or []:
         callback.on_hyperopt_init(experiment_name)
 
-    if not should_tune_preprocessing(config):
+    if not should_tune_preprocessing(full_config):
         # preprocessing is not being tuned, so generate it once before starting trials
         for callback in callbacks or []:
             callback.on_hyperopt_preprocessing_start(experiment_name)
 
         model = LudwigModel(
-            config=config,
+            config=full_config,
             backend=backend,
             gpus=gpus,
             gpu_memory_limit=gpu_memory_limit,
@@ -383,6 +371,7 @@ def hyperopt(
         gpu_memory_limit=gpu_memory_limit,
         allow_parallel_threads=allow_parallel_threads,
         callbacks=callbacks,
+        tune_callbacks=tune_callbacks,
         backend=backend,
         random_seed=random_seed,
         hyperopt_log_verbosity=hyperopt_log_verbosity,
