@@ -1,4 +1,3 @@
-import copy
 import logging
 import os
 from typing import Dict, Tuple, Union
@@ -8,13 +7,15 @@ import torch
 import torchmetrics
 
 from ludwig.combiners.combiners import get_combiner_class
-from ludwig.constants import MODEL_ECD, TYPE
+from ludwig.constants import MODEL_ECD
 from ludwig.globals import MODEL_WEIGHTS_FILE_NAME
 from ludwig.models.base import BaseModel
-from ludwig.schema.utils import load_config_with_kwargs
+from ludwig.schema.model_config import ModelConfig
 from ludwig.utils import output_feature_utils
+from ludwig.utils.augmentation_utils import AugmentationPipelines
 from ludwig.utils.data_utils import clear_data_cache
 from ludwig.utils.fs_utils import open_file
+from ludwig.utils.state_dict_backward_compatibility import update_state_dict
 from ludwig.utils.torch_utils import get_torch_device
 
 logger = logging.getLogger(__name__)
@@ -27,39 +28,32 @@ class ECD(BaseModel):
 
     def __init__(
         self,
-        input_features,
-        combiner,
-        output_features,
+        config_obj: ModelConfig,
         random_seed=None,
         **_kwargs,
     ):
-        self._input_features_def = copy.deepcopy(input_features)
-        self._combiner_def = copy.deepcopy(combiner)
-        self._output_features_def = copy.deepcopy(output_features)
-
+        self.config_obj = config_obj
         self._random_seed = random_seed
 
         super().__init__(random_seed=self._random_seed)
 
         # ================ Inputs ================
         try:
-            self.input_features.update(self.build_inputs(self._input_features_def))
+            self.input_features.update(self.build_inputs(input_feature_configs=self.config_obj.input_features))
         except KeyError as e:
             raise KeyError(
                 f"An input feature has a name that conflicts with a class attribute of torch's ModuleDict: {e}"
             )
 
         # ================ Combiner ================
-        logger.debug(f"Combiner {self._combiner_def[TYPE]}")
-        combiner_class = get_combiner_class(self._combiner_def[TYPE])
-        config, kwargs = load_config_with_kwargs(
-            combiner_class.get_schema_cls(),
-            self._combiner_def,
-        )
-        self.combiner = combiner_class(input_features=self.input_features, config=config, **kwargs)
+        logger.debug(f"Combiner {self.config_obj.combiner.type}")
+        combiner_class = get_combiner_class(self.config_obj.combiner.type)
+        self.combiner = combiner_class(input_features=self.input_features, config=self.config_obj.combiner)
 
         # ================ Outputs ================
-        self.output_features.update(self.build_outputs(self._output_features_def, self.combiner))
+        self.output_features.update(
+            self.build_outputs(output_feature_configs=self.config_obj.output_features, combiner=self.combiner)
+        )
 
         # ================ Combined loss metric ================
         self.eval_loss_metric = torchmetrics.MeanMetric()
@@ -148,6 +142,10 @@ class ECD(BaseModel):
         combiner_outputs = self.combine(encoder_outputs)
         return self.decode(combiner_outputs, targets, mask)
 
+    def unskip(self):
+        for k in self.input_features.keys():
+            self.input_features[k] = self.input_features[k].unskip()
+
     def save(self, save_path):
         """Saves the model to the given path."""
         weights_save_path = os.path.join(save_path, MODEL_WEIGHTS_FILE_NAME)
@@ -158,8 +156,30 @@ class ECD(BaseModel):
         weights_save_path = os.path.join(save_path, MODEL_WEIGHTS_FILE_NAME)
         device = torch.device(get_torch_device())
         with open_file(weights_save_path, "rb") as f:
-            self.load_state_dict(torch.load(f, map_location=device))
+            state_dict = torch.load(f, map_location=device)
+            self.load_state_dict(update_state_dict(state_dict))
 
     def get_args(self):
         """Returns init arguments for constructing this model."""
-        return (self._input_features_df, self._combiner_def, self._output_features_df, self._random_seed)
+        return (
+            self.config_obj.input_features.to_list(),
+            self.config_obj.combiner.to_dict(),
+            self.config_obj.output_features.to_list(),
+            self._random_seed,
+        )
+
+    def get_augmentation_pipelines(self) -> AugmentationPipelines:
+        """Returns the augmentation pipeline for this model."""
+        # dictionary to hold any augmentation pipeline
+        augmentation_pipelines = {}
+
+        # loop through all input features and add their augmentation pipeline to the dictionary
+        for input_feature in self.config_obj.input_features:
+            # if augmentation was specified for this input feature, add AugmentationPipeline to dictionary
+            if input_feature.has_augmentation():
+                # use input feature proc_column as key because that is what is used in the Batcher
+                augmentation_pipelines[input_feature.proc_column] = self.input_features[
+                    input_feature.name
+                ].get_augmentation_pipeline()
+
+        return AugmentationPipelines(augmentation_pipelines)
