@@ -20,12 +20,13 @@ from typing import Any, Dict, List, Optional, Union
 
 import torch
 
+from ludwig.constants import PADDING_SYMBOL, UNKNOWN_SYMBOL
 from ludwig.utils.data_utils import load_json
+from ludwig.utils.hf_utils import load_pretrained_hf_tokenizer
 from ludwig.utils.nlp_utils import load_nlp_pipeline, process_text
 
 logger = logging.getLogger(__name__)
 
-
 SPACE_PUNCTUATION_REGEX = re.compile(r"\w+|[^\w\s]")
 COMMA_REGEX = re.compile(r"\s*,\s*")
 UNDERSCORE_REGEX = re.compile(r"\s*_\s*")
@@ -34,14 +35,7 @@ TORCHSCRIPT_COMPATIBLE_TOKENIZERS = {"space", "space_punct"}
 TORCHTEXT_0_12_0_TOKENIZERS = {"sentencepiece", "clip", "gpt2bpe"}
 TORCHTEXT_0_13_0_TOKENIZERS = {"bert"}
 
-# Do not use torchtext implementation of BERT tokenizer for these model names:
-# https://github.com/pytorch/text/issues/1840
-SKIP_TORCHTEXT_BERT_HF_MODEL_NAMES = {
-    "bert-base-german-cased",
-    "bert-base-german-dbmdz-cased",
-    "bert-base-german-dbmdz-uncased",
-    "TurkuNLP/bert-base-finnish-cased-v1",
-}
+HF_TOKENIZER_SAMPLE_INPUTS = ["UNwant\u00E9d,running", "ah\u535A\u63A8zz", " \tHeLLo!how  \n Are yoU? [UNK]"]
 
 
 class BaseTokenizer:
@@ -80,13 +74,29 @@ class SpaceStringToListTokenizer(torch.nn.Module):
         for sequence in inputs:
             split_sequence = sequence.strip().split(" ")
             token_sequence: List[str] = []
-            for token in split_sequence:
+            for token in self.get_tokens(split_sequence):
                 if len(token) > 0:
                     token_sequence.append(token)
             tokens.append(token_sequence)
 
         return tokens[0] if isinstance(v, str) else tokens
 
+    def get_tokens(self, tokens: List[str]) -> List[str]:
+        return tokens
+
+
+class NgramTokenizer(SpaceStringToListTokenizer):
+    """Implements torchscript-compatible n-gram tokenization."""
+
+    def __init__(self, ngram_size: int = 2, **kwargs):
+        super().__init__()
+        self.n = ngram_size or 2
+
+    def get_tokens(self, tokens: List[str]) -> List[str]:
+        from torchtext.data.utils import ngrams_iterator
+
+        return list(ngrams_iterator(tokens, ngrams=self.n))
+
 
 class SpacePunctuationStringToListTokenizer(torch.nn.Module):
     """Implements torchscript-compatible space_punct tokenization."""
@@ -779,11 +789,8 @@ class MultiLemmatizeRemoveStopwordsTokenizer(BaseTokenizer):
 class HFTokenizer(BaseTokenizer):
     def __init__(self, pretrained_model_name_or_path, **kwargs):
         super().__init__()
-        from transformers import AutoTokenizer
-
-        self.tokenizer = AutoTokenizer.from_pretrained(
-            pretrained_model_name_or_path,
-        )
+        self.pretrained_model_name_or_path = pretrained_model_name_or_path
+        self.tokenizer = load_pretrained_hf_tokenizer(self.pretrained_model_name_or_path)
 
     def __call__(self, text):
         return self.tokenizer.encode(text, truncation=True)
@@ -792,6 +799,11 @@ class HFTokenizer(BaseTokenizer):
         return self.tokenizer.get_vocab()
 
     def get_pad_token(self) -> str:
+        # HACK(geoffrey): gpt2 has no pad token. Recommendation is to use eos token instead.
+        # https://github.com/huggingface/transformers/issues/2630#issuecomment-1290809338
+        # https://github.com/huggingface/transformers/issues/2648#issuecomment-616177044
+        if self.pretrained_model_name_or_path == "gpt2":
+            self.tokenizer.pad_token = self.tokenizer.eos_token
         return self.tokenizer.pad_token
 
     def get_unk_token(self) -> str:
@@ -802,6 +814,7 @@ tokenizer_registry = {
     # Torchscript-compatible tokenizers. Torchtext tokenizers are also available below (requires torchtext>=0.12.0).
     "space": SpaceStringToListTokenizer,
     "space_punct": SpacePunctuationStringToListTokenizer,
+    "ngram": NgramTokenizer,
     # Tokenizers not compatible with torchscript
     "characters": CharactersToListTokenizer,
     "underscore": UnderscoreStringToListTokenizer,
@@ -937,7 +950,6 @@ try:
             def __init__(self, pretrained_model_name_or_path: str, vocab_file: str):
                 super().__init__()
                 self.str2idx, self.idx2str = self._init_vocab(vocab_file)
-                # TODO(geoffrey): If we move to torchtext>=0.13.0, we can use return_tokens kwarg to get tokens directly
                 self.tokenizer = self._init_tokenizer(pretrained_model_name_or_path, vocab_file)
 
             def _init_vocab(self, vocab_file: str) -> Dict[str, str]:
@@ -1014,65 +1026,87 @@ try:
         )
         TORCHSCRIPT_COMPATIBLE_TOKENIZERS.update(TORCHTEXT_0_12_0_TOKENIZERS)
     else:
-        raise ImportError
+        raise ImportError(f"torchtext>=0.12.0 is required to use these tokenizers: {TORCHTEXT_0_12_0_TOKENIZERS}.")
 
 except ImportError:
-    logger.warning(
-        f"torchtext>=0.12.0 is not installed, so the following tokenizers are not available: "
-        f"{TORCHTEXT_0_12_0_TOKENIZERS}"
-    )
+    pass
 
 
 try:
     import torchtext
 
-    if torch.torch_version.TorchVersion(torchtext.__version__) >= (0, 13, 0):
+    torchtext_version = torch.torch_version.TorchVersion(torchtext.__version__)
+
+    if torchtext_version >= (0, 13, 0):
         pass
     else:
-        raise ImportError
+        raise ImportError(f"torchtext>=0.13.0 is required to use these tokenizers: {TORCHTEXT_0_13_0_TOKENIZERS}.")
 
     class BERTTokenizer(torch.nn.Module):
         def __init__(
             self,
             vocab_file: Optional[str] = None,
             is_hf_tokenizer: Optional[bool] = False,
-            do_lower_case: Optional[bool] = False,
+            hf_tokenizer_attrs: Optional[Dict[str, Any]] = None,
             **kwargs,
         ):
             super().__init__()
-            from transformers.utils.hub import cached_path
 
             if vocab_file is None:
-                # By default, we use the uncased (all lowercased) BERT tokenizer from huggingface.
-                vocab_file = "https://huggingface.co/bert-base-uncased/resolve/main/vocab.txt"
-                do_lower_case = True
-            self.vocab_file = cached_path(vocab_file)
+                # If vocab_file not passed in, use default "bert-base-uncased" vocab and kwargs.
+                kwargs = _get_bert_config("bert-base-uncased")
+                vocab_file = kwargs["vocab_file"]
+                vocab = self._init_vocab(vocab_file)
+                hf_tokenizer_attrs = {
+                    "pad_token": "[PAD]",
+                    "unk_token": "[UNK]",
+                    "sep_token_id": vocab["[SEP]"],
+                    "cls_token_id": vocab["[CLS]"],
+                }
+            else:
+                vocab = self._init_vocab(vocab_file)
+
+            self.vocab = vocab
 
             self.is_hf_tokenizer = is_hf_tokenizer
+            if self.is_hf_tokenizer:
+                # Values used by Ludwig extracted from the corresponding HF model.
+                self.pad_token = hf_tokenizer_attrs["pad_token"]  # Used as padding symbol
+                self.unk_token = hf_tokenizer_attrs["unk_token"]  # Used as unknown symbol
+                self.cls_token_id = hf_tokenizer_attrs["cls_token_id"]  # Used as start symbol. Only used if HF.
+                self.sep_token_id = hf_tokenizer_attrs["sep_token_id"]  # Used as stop symbol. Only used if HF.
+                self.never_split = hf_tokenizer_attrs["all_special_tokens"]
+            else:
+                self.pad_token = PADDING_SYMBOL
+                self.unk_token = UNKNOWN_SYMBOL
+                self.cls_token_id = None
+                self.sep_token_id = None
+                self.never_split = [UNKNOWN_SYMBOL]
+
+            tokenizer_kwargs = {}
+            if "do_lower_case" in kwargs:
+                tokenizer_kwargs["do_lower_case"] = kwargs["do_lower_case"]
+            if "strip_accents" in kwargs:
+                tokenizer_kwargs["strip_accents"] = kwargs["strip_accents"]
 
             # Return tokens as raw tokens only if not being used as a HF tokenizer.
             self.return_tokens = not self.is_hf_tokenizer
-            self.tokenizer = torchtext.transforms.BERTTokenizer(
-                vocab_path=self.vocab_file, return_tokens=self.return_tokens, do_lower_case=do_lower_case
-            )
 
-            self.str2idx = self._init_vocab(self.vocab_file)
+            tokenizer_init_kwargs = {
+                **tokenizer_kwargs,
+                "vocab_path": vocab_file,
+                "return_tokens": self.return_tokens,
+            }
+            if torchtext_version >= (0, 14, 0):
+                # never_split kwarg added in torchtext 0.14.0
+                tokenizer_init_kwargs["never_split"] = self.never_split
 
-            # Values from
-            # https://github.com/huggingface/transformers/blob/main/src/transformers/models/bert/tokenization_bert.py
-            self.pad_token = "[PAD]"
-            self.unk_token = "[UNK]"
-            self.cls_token_id = self.str2idx["[CLS]"]  # Used as start symbol
-            self.sep_token_id = self.str2idx["[SEP]"]  # Used as stop symbol
+            self.tokenizer = torchtext.transforms.BERTTokenizer(**tokenizer_init_kwargs)
 
-        def _init_vocab(self, vocab_file: str) -> Dict[str, str]:
-            str2idx = {}
-            with open(vocab_file, encoding="utf-8") as reader:
-                tokens = reader.readlines()
-            for index, token in enumerate(tokens):
-                token = token.rstrip("\n")
-                str2idx[token] = index
-            return str2idx
+        def _init_vocab(self, vocab_file: str) -> Dict[str, int]:
+            from transformers.models.bert.tokenization_bert import load_vocab
+
+            return load_vocab(vocab_file)
 
         def forward(self, v: Union[str, List[str], torch.Tensor]) -> Any:
             """Implements forward pass for tokenizer.
@@ -1100,8 +1134,7 @@ try:
                 token_ids: List[List[int]] = []
                 for token_ids_str_i in token_ids_str:
                     token_ids_i = [int(token_id_str) for token_id_str in token_ids_str_i]
-                    token_ids_i.insert(0, self.cls_token_id)
-                    token_ids_i.append(self.sep_token_id)
+                    token_ids_i = self._add_special_token_ids(token_ids_i)
                     token_ids.append(token_ids_i)
                 return token_ids[0] if isinstance(v, str) else token_ids
 
@@ -1109,8 +1142,8 @@ try:
             assert torch.jit.isinstance(tokens, List[List[str]])
             return tokens[0] if isinstance(v, str) else tokens
 
-        def get_vocab(self) -> Dict[str, str]:
-            return self.str2idx
+        def get_vocab(self) -> Dict[str, int]:
+            return self.vocab
 
         def get_pad_token(self) -> str:
             return self.pad_token
@@ -1118,6 +1151,13 @@ try:
         def get_unk_token(self) -> str:
             return self.unk_token
 
+        def _add_special_token_ids(self, token_ids: List[int]) -> List[int]:
+            """Adds special token ids to the token_ids list."""
+            if torch.jit.isinstance(self.cls_token_id, int) and torch.jit.isinstance(self.sep_token_id, int):
+                token_ids.insert(0, self.cls_token_id)
+                token_ids.append(self.sep_token_id)
+            return token_ids
+
     tokenizer_registry.update(
         {
             "bert": BERTTokenizer,
@@ -1126,10 +1166,7 @@ try:
     TORCHSCRIPT_COMPATIBLE_TOKENIZERS.update(TORCHTEXT_0_13_0_TOKENIZERS)
 
 except ImportError:
-    logger.warning(
-        f"torchtext>=0.13.0 is not installed, so the following tokenizers are not available: "
-        f"{TORCHTEXT_0_13_0_TOKENIZERS}"
-    )
+    pass
 
 
 def get_hf_tokenizer(pretrained_model_name_or_path, **kwargs):
@@ -1140,27 +1177,72 @@ def get_hf_tokenizer(pretrained_model_name_or_path, **kwargs):
     Returns:
         A torchscript-able HF tokenizer if it is available. Else, returns vanilla HF tokenizer.
     """
+    from transformers import BertTokenizer
 
-    if "bert" in TORCHSCRIPT_COMPATIBLE_TOKENIZERS:
-        from transformers.models.bert.tokenization_bert import PRETRAINED_INIT_CONFIGURATION, PRETRAINED_VOCAB_FILES_MAP
+    hf_name = pretrained_model_name_or_path
+    # use_fast=False to leverage python class inheritance
+    # cannot tokenize HF tokenizers directly because HF lacks strict typing and List[str] cannot be traced
+    hf_tokenizer = load_pretrained_hf_tokenizer(hf_name, use_fast=False)
 
-        if (
-            pretrained_model_name_or_path in PRETRAINED_VOCAB_FILES_MAP["vocab_file"]
-            and pretrained_model_name_or_path not in SKIP_TORCHTEXT_BERT_HF_MODEL_NAMES
-        ):
-            logger.info(f"Loading TorchText implementation of {pretrained_model_name_or_path} tokenizer")
-            vocab_file = PRETRAINED_VOCAB_FILES_MAP["vocab_file"][pretrained_model_name_or_path]
-            init_kwargs = PRETRAINED_INIT_CONFIGURATION.get(pretrained_model_name_or_path, {})
-            return BERTTokenizer(
-                vocab_file,
-                is_hf_tokenizer=True,
-                **init_kwargs,
-            )
+    torchtext_tokenizer = None
+    if "bert" in TORCHSCRIPT_COMPATIBLE_TOKENIZERS and isinstance(hf_tokenizer, BertTokenizer):
+        tokenizer_kwargs = _get_bert_config(hf_name)
+        torchtext_tokenizer = BERTTokenizer(
+            **tokenizer_kwargs,
+            is_hf_tokenizer=True,
+            hf_tokenizer_attrs={
+                "pad_token": hf_tokenizer.pad_token,
+                "unk_token": hf_tokenizer.unk_token,
+                "cls_token_id": hf_tokenizer.cls_token_id,
+                "sep_token_id": hf_tokenizer.sep_token_id,
+                "all_special_tokens": hf_tokenizer.all_special_tokens,
+            },
+        )
 
-    # If pretrained_model_name_or_path does not have a torchtext equivalent implementation, load the
-    # HuggingFace implementation.
-    logger.info(f"Loading HuggingFace implementation of {pretrained_model_name_or_path} tokenizer")
-    return HFTokenizer(pretrained_model_name_or_path)
+    use_torchtext = torchtext_tokenizer is not None
+    if use_torchtext:
+        # If a torchtext tokenizer is instantiable, tenatively we will use it. However,
+        # if the tokenizer does not pass (lightweight) validation, then we will fall back to the vanilla HF tokenizer.
+        # TODO(geoffrey): can we better validate tokenizer parity before swapping in the TorchText tokenizer?
+        # Samples from https://github.com/huggingface/transformers/blob/main/tests/models/bert/test_tokenization_bert.py
+        for sample_input in HF_TOKENIZER_SAMPLE_INPUTS:
+            hf_output = hf_tokenizer.encode(sample_input)
+            tt_output = torchtext_tokenizer(sample_input)
+            if hf_output != tt_output:
+                use_torchtext = False
+                logger.warning("Falling back to HuggingFace tokenizer because TorchText tokenizer failed validation.")
+                logger.warning(f"Sample input: {sample_input}\nHF output: {hf_output}\nTT output: {tt_output}")
+                break
+
+    if use_torchtext:
+        logger.info(f"Loaded TorchText implementation of {hf_name} tokenizer")
+        return torchtext_tokenizer
+    else:
+        # If hf_name does not have a torchtext equivalent implementation, load the
+        # HuggingFace implementation.
+        logger.info(f"Loaded HuggingFace implementation of {hf_name} tokenizer")
+        return HFTokenizer(hf_name)
+
+
+def _get_bert_config(hf_name):
+    """Gets configs from BERT tokenizers in HuggingFace.
+
+    `vocab_file` is required for BERT tokenizers. `tokenizer_config.json` are optional keyword arguments used to
+    initialize the tokenizer object. If no `tokenizer_config.json` is found, then we instantiate the tokenizer with
+    default arguments.
+    """
+    from transformers.utils.hub import cached_path, EntryNotFoundError
+
+    vocab_file = cached_path(f"https://huggingface.co/{hf_name}/resolve/main/vocab.txt")
+
+    try:
+        tokenizer_config = load_json(
+            cached_path(f"https://huggingface.co/{hf_name}/resolve/main/tokenizer_config.json")
+        )
+    except EntryNotFoundError:
+        tokenizer_config = {}
+
+    return {"vocab_file": vocab_file, **tokenizer_config}
 
 
 tokenizer_registry.update(
@@ -1168,3 +1250,34 @@ tokenizer_registry.update(
         "hf_tokenizer": get_hf_tokenizer,
     }
 )
+
+
+def get_tokenizer_from_registry(tokenizer_name: str) -> torch.nn.Module:
+    """Returns the appropriate tokenizer from the tokenizer registry.
+
+    Raises a KeyError if a tokenizer that does not exist in the registry is requested, with additional help text if the
+    requested tokenizer would be available for a different version of torchtext.
+    """
+    if tokenizer_name in tokenizer_registry:
+        return tokenizer_registry[tokenizer_name]
+
+    if (
+        torch.torch_version.TorchVersion(torchtext.__version__) < (0, 12, 0)
+        and tokenizer_name in TORCHTEXT_0_12_0_TOKENIZERS
+    ):
+        raise KeyError(
+            f"torchtext>=0.12.0 is not installed, so '{tokenizer_name}' and the following tokenizers are not "
+            f"available: {TORCHTEXT_0_12_0_TOKENIZERS}"
+        )
+
+    if (
+        torch.torch_version.TorchVersion(torchtext.__version__) < (0, 13, 0)
+        and tokenizer_name in TORCHTEXT_0_13_0_TOKENIZERS
+    ):
+        raise KeyError(
+            f"torchtext>=0.13.0 is not installed, so '{tokenizer_name}' and the following tokenizers are not "
+            f"available: {TORCHTEXT_0_13_0_TOKENIZERS}"
+        )
+
+    # Tokenizer does not exist or is unavailable.
+    raise KeyError(f"Invalid tokenizer name: '{tokenizer_name}'. Available tokenizers: {tokenizer_registry.keys()}")
