@@ -4,12 +4,13 @@ import re
 import shutil
 import tempfile
 from dataclasses import dataclass
-from typing import Any, Dict, List, Optional, Tuple, Union
+from typing import Dict, List, Optional, Tuple, Union
 
 import pandas as pd
 import torch
 
 from ludwig.api import LudwigModel
+from ludwig.api_annotations import DeveloperAPI
 from ludwig.constants import (
     AUDIO,
     BAG,
@@ -35,6 +36,7 @@ from ludwig.models.inference import (
     _InferencePreprocessor,
     InferenceModule,
 )
+from ludwig.types import ModelConfigDict
 from ludwig.utils.inference_utils import to_inference_module_input_from_dataframe
 from ludwig.utils.misc_utils import remove_empty_lines
 from ludwig.utils.torch_utils import model_size, place_on_device
@@ -184,6 +186,7 @@ def to_triton_type(content: Union[List[str], List[torch.Tensor], List[TorchAudio
         return _get_type_map(str(content.dtype))
 
 
+@DeveloperAPI
 @dataclass
 class TritonArtifact:
     """Dataclass for exported Triton artifacts."""
@@ -207,6 +210,7 @@ class TritonArtifact:
     content_length: int
 
 
+@DeveloperAPI
 @dataclass
 class TritonConfigFeature:
     """Represents an input/output feature in a Triton config."""
@@ -268,6 +272,7 @@ class TritonConfigFeature:
         return "torch.Tensor"
 
 
+@DeveloperAPI
 @dataclass
 class TritonMaster:
     """Provides access to the Triton Config and the scripted module."""
@@ -297,7 +302,7 @@ class TritonMaster:
     model_version: Union[int, str]
 
     # Ludwig config.
-    ludwig_config: Dict[str, Any]
+    ludwig_config: ModelConfigDict
 
     # One of "cpu", "cuda".
     device: str
@@ -408,6 +413,7 @@ class TritonMaster:
         return config_artifact
 
 
+@DeveloperAPI
 @dataclass
 class TritonEnsembleConfig:
     """Dataclass for creating and saving the Triton ensemble config."""
@@ -523,6 +529,7 @@ class TritonEnsembleConfig:
         return model_artifact
 
 
+@DeveloperAPI
 @dataclass
 class TritonConfig:
     """Enables the creation and export of a Triton config."""
@@ -599,6 +606,7 @@ class TritonConfig:
         return config
 
 
+@DeveloperAPI
 @dataclass
 class TritonModel:
     """Enables the scripting and export of a model."""
@@ -696,13 +704,14 @@ def get_example_input(
 
     Generates a synthetic example if one is not provided.
     """
+    config = model.config
     if data_example is None:
-        features = model.config["input_features"] + model.config["output_features"]
+        features = config["input_features"] + config["output_features"]
         df = build_synthetic_dataset(dataset_size=1, features=features)
         data = [row for row in df]
         data_example = pd.DataFrame(data[1:], columns=data[0])
     return to_inference_module_input_from_dataframe(
-        data_example.head(1), model.config, load_paths=True, device=device_types[0]
+        data_example.head(1), config, load_paths=True, device=device_types[0]
     )
 
 
@@ -712,6 +721,7 @@ def clean_up_synthetic_data():
     shutil.rmtree("image_files", ignore_errors=True)
 
 
+@DeveloperAPI
 def export_triton(
     model: LudwigModel,
     data_example: Optional[pd.DataFrame] = None,
