@@ -17,6 +17,7 @@ import copy
 import logging
 from collections import Counter
 from sys import platform
+from typing import Dict, List
 
 import numpy as np
 import pandas as pd
@@ -34,7 +35,7 @@ try:
         try:
             mpl.use("TkAgg")
         except ModuleNotFoundError:
-            logging.warning("Unable to set TkAgg backend for matplotlib. Your Python may not be configured for Tk")
+            logger.warning("Unable to set TkAgg backend for matplotlib. Your Python may not be configured for Tk")
     import matplotlib.patches as patches
     import matplotlib.path as path
     import matplotlib.patheffects as PathEffects
@@ -61,11 +62,6 @@ RAY_TUNE_CATEGORY_SPACES = {"choice", "grid_search"}
 _matplotlib_34 = version.parse(mpl.__version__) >= version.parse("3.4")
 
 
-# plt.rc('xtick', labelsize='x-large')
-# plt.rc('ytick', labelsize='x-large')
-# plt.rc('axes', labelsize='x-large')
-
-
 def visualize_callbacks(callbacks, fig):
     if callbacks is None:
         return
@@ -74,7 +70,15 @@ def visualize_callbacks(callbacks, fig):
 
 
 def learning_curves_plot(
-    train_values, vali_values, metric, algorithm_names=None, title=None, filename=None, callbacks=None
+    train_values,
+    vali_values,
+    metric,
+    x_label="epoch",
+    x_step=1,
+    algorithm_names=None,
+    title=None,
+    filename=None,
+    callbacks=None,
 ):
     num_algorithms = len(train_values)
     max_len = max(len(tv) for tv in train_values)
@@ -94,10 +98,10 @@ def learning_curves_plot(
     ax.grid(which="both")
     ax.grid(which="minor", alpha=0.5)
     ax.grid(which="major", alpha=0.75)
-    ax.set_xlabel("epochs")
+    ax.set_xlabel(x_label)
     ax.set_ylabel(metric.replace("_", " "))
 
-    xs = list(range(1, max_len + 1))
+    xs = np.arange(1, (max_len * x_step) + 1, x_step)
 
     for i in range(num_algorithms):
         name_prefix = algorithm_names[i] + " " if algorithm_names is not None and i < len(algorithm_names) else ""
@@ -896,10 +900,62 @@ def roc_curves(
         plt.show()
 
 
+def precision_recall_curves_plot(
+    precision_recalls: Dict[str, List[float]],
+    model_names: List[str],
+    title: str = None,
+    filename: str = None,
+    callbacks=None,
+):
+    """Generates a precision recall curve for each model in the model_names list.
+
+    Args:
+        precision_recalls: A list of dictionaries representing the precision and recall values for each model
+            in model_names. Each dictionary has two keys: "precisions" and "recalls".
+    """
+    sns.set_style("whitegrid")
+
+    colors = plt.get_cmap("tab10").colors
+
+    _, ax = plt.subplots()
+
+    ax.set_xlim(0, 1)
+    # Create ticks for every 0.1 increment
+    ax.set_xticks(np.linspace(0, 1, 11))
+    ax.set_xlabel("Recall")
+
+    ax.set_ylim(0, 1)
+    # Create ticks for every 0.1 increment
+    ax.set_yticks(np.linspace(0, 1, 11))
+    ax.set_ylabel("Precision")
+
+    if title is not None:
+        ax.set_title(title)
+
+    for i in range(len(precision_recalls)):
+        model_name = model_names[i] if model_names is not None and i < len(model_names) else ""
+        ax.plot(
+            precision_recalls[i]["recalls"],
+            precision_recalls[i]["precisions"],
+            label=model_name,
+            color=colors[i],
+            linewidth=3,
+        )
+
+    ax.legend(frameon=True)
+    plt.tight_layout()
+    visualize_callbacks(callbacks, plt.gcf())
+    if filename:
+        plt.savefig(filename)
+    else:
+        plt.show()
+
+
 def calibration_plot(
     fraction_positives,
     mean_predicted_values,
     algorithm_names=None,
+    class_name=None,
     filename=None,
     callbacks=None,
 ):
@@ -935,7 +991,7 @@ def calibration_plot(
             color=colors[i],
             marker="o",
             scatter_kws={"s": 40},
-            label=algorithm_names[i] if algorithm_names is not None and i < len(algorithm_names) else "",
+            label=algorithm_names[i] if algorithm_names is not None and i < len(algorithm_names) else f"Model {i}",
         )
 
     ticks = np.linspace(0.0, 1.0, num=11)
@@ -946,7 +1002,10 @@ def calibration_plot(
     plt.ylim([-0.05, 1.05])
     plt.yticks(ticks)
     plt.legend(loc="lower right")
-    plt.title("Calibration (reliability curve)")
+    if class_name is not None:
+        plt.title(f"{class_name}: Calibration (reliability curve)")
+    else:
+        plt.title("Calibration (reliability curve)")
 
     plt.tight_layout()
     visualize_callbacks(callbacks, plt.gcf())
@@ -959,34 +1018,60 @@ def calibration_plot(
 def brier_plot(
     brier_scores,
     algorithm_names=None,
+    class_names=None,
     title=None,
     filename=None,
     callbacks=None,
 ):
-    plt.figure()
     sns.set_style("whitegrid")
 
+    # Dynamically set the size of the plot based on the number of labels
+    # Use minimum size to prevent plot from being too small
+    default_width, default_height = plt.rcParams.get("figure.figsize")
+    width = max(default_width, len(class_names) / 2)
+    height = max(default_height, len(class_names) / 2)
+    fig, ax = plt.subplots(figsize=(width, height))
+
     if title is not None:
         plt.title(title)
 
     colors = plt.get_cmap("tab10").colors
 
-    plt.grid(which="both")
-    plt.grid(which="minor", alpha=0.5)
-    plt.grid(which="major", alpha=0.75)
-    plt.xlabel("class")
-    plt.ylabel("brier")
+    n_algorithms = brier_scores.shape[1]
+    n_classes = brier_scores.shape[0]
+    x = np.arange(n_classes)
 
-    for i in range(brier_scores.shape[1]):
-        plt.plot(
-            brier_scores[:, i],
-            label=algorithm_names[i] + " " if algorithm_names is not None and i < len(algorithm_names) else "",
-            color=colors[i],
-            linewidth=3,
+    max_width = 0.35
+    bar_width = min(0.5 / n_algorithms, max_width)
+    bar_left = -bar_width * (n_algorithms // 2) + ((bar_width / 2) if (n_algorithms % 2) == 0 else 0)
+
+    ax.grid(which="both")
+    ax.grid(which="minor", alpha=0.5)
+    ax.grid(which="major", alpha=0.75)
+    ax.set_xlabel("class")
+    ax.set_ylabel("brier score")
+    if class_names is not None:
+        ax.set_xticks(
+            x,
+            class_names,
+            rotation=45,
+            ha="center",
+        )
+    else:
+        ax.set_xticks(
+            x,
+            [str(i) for i in range(n_classes)],
+            rotation=45,
+            ha="center",
         )
 
-    plt.legend()
-    plt.tight_layout()
+    for i in range(n_algorithms):
+        # Plot bar for each class
+        label = algorithm_names[i] if algorithm_names is not None and i < len(algorithm_names) else f"Model {i}"
+        ax.bar(x + bar_left + (bar_width * i), brier_scores[:, i], bar_width, color=colors[i], label=label)
+
+    ax.legend()
+    fig.tight_layout()
     visualize_callbacks(callbacks, plt.gcf())
     if filename:
         plt.savefig(filename)
@@ -1045,28 +1130,53 @@ def confusion_matrix_plot(
     callbacks=None,
 ):
     mpl.rcParams.update({"figure.autolayout": True})
-    fig, ax = plt.subplots()
+
+    # Dynamically set the size of the plot based on the number of labels
+    # Use minimum size to prevent plot from being too small
+    default_width, default_height = plt.rcParams.get("figure.figsize")
+    width = max(default_width, len(labels))
+    height = max(default_height, len(labels))
+    fig, ax = plt.subplots(figsize=(width, height))
 
     ax.invert_yaxis()
     ax.xaxis.tick_top()
     ax.xaxis.set_label_position("top")
 
-    cax = ax.matshow(confusion_matrix, cmap="viridis")
+    # Set alpha value to prevent blue hues from being too dark
+    cax = ax.matshow(confusion_matrix, cmap="Blues", alpha=0.6)
+    # Annotate confusion matrix plot
+    for (i, j), z in np.ndenumerate(confusion_matrix):
+        # Format differently based on whether the value is normalized or not
+        if z.is_integer():
+            z_format = f"{z:.0f}"
+        else:
+            z_format = f"{z:.3f}"
+        ax.text(
+            j,
+            i,
+            z_format,
+            ha="center",
+            va="center",
+            color="black",
+            fontweight="medium",
+            wrap=True,
+        )
 
     ax.xaxis.set_major_locator(ticker.MultipleLocator(1))
     ax.yaxis.set_major_locator(ticker.MultipleLocator(1))
     ax.set_xticklabels([""] + labels, rotation=45, ha="left")
-    ax.set_yticklabels([""] + labels)
+    ax.set_yticklabels([""] + labels, rotation=45, ha="right")
     ax.grid(False)
     ax.tick_params(axis="both", which="both", length=0)
-    fig.colorbar(cax, ax=ax, extend="max")
+    # https://stackoverflow.com/a/26720422/10102370 works nicely for square plots
+    fig.colorbar(cax, ax=ax, extend="max", fraction=0.046, pad=0.04)
     ax.set_xlabel(f"Predicted {output_feature_name}")
     ax.set_ylabel(f"Actual {output_feature_name}")
 
     plt.tight_layout()
     visualize_callbacks(callbacks, plt.gcf())
     if filename:
-        plt.savefig(filename)
+        plt.savefig(filename, bbox_inches="tight")
     else:
         plt.show()
 
@@ -1085,15 +1195,15 @@ def double_axis_line_plot(
 
     colors = plt.get_cmap("tab10").colors
 
-    fig, ax1 = plt.subplots()
+    # Dynamically adjust figure size based on number of labels
+    default_width, default_height = plt.rcParams.get("figure.figsize")
+    width = max(default_width, len(labels) / 3)
+    height = max(default_height, len(labels) / 3)
+    fig, ax1 = plt.subplots(layout="constrained", figsize=(width, height))
 
     if title is not None:
         ax1.set_title(title)
 
-    # ax1.grid(which='both')
-    # ax1.grid(which='minor', alpha=0.5)
-    # ax1.grid(which='major', alpha=0.75)
-
     ax1.set_xlabel(f"class (sorted by {y1_name})")
     ax1.set_xlim(0, len(y1_sorted) - 1)
     if labels is not None:
@@ -1226,7 +1336,11 @@ def bar_plot(
 
     sns.set_style("whitegrid")
 
-    fig, ax = plt.subplots()
+    # Dynamically set the size of the plot based on the number of labels
+    # Use minimum size to prevent plot from being too small
+    default_width, default_height = plt.rcParams.get("figure.figsize")
+    width = max(default_width, len(labels) / 2)
+    _, ax = plt.subplots(figsize=(width, default_height))
 
     ax.grid(which="both")
     ax.grid(which="minor", alpha=0.5)
