@@ -25,7 +25,7 @@ import yaml
 
 from ludwig.api import LudwigModel
 from ludwig.backend import LOCAL_BACKEND
-from ludwig.constants import ENCODER, H3, TRAINER, TYPE
+from ludwig.constants import BATCH_SIZE, ENCODER, H3, PREPROCESSING, TRAINER, TYPE
 from ludwig.data.concatenate_datasets import concatenate_df
 from ludwig.data.preprocessing import preprocess_for_training
 from ludwig.encoders.registry import get_encoder_classes
@@ -45,15 +45,12 @@ from tests.integration_tests.utils import (
     generate_output_features_with_dependencies,
     generate_output_features_with_dependencies_complex,
     h3_feature,
-    HF_ENCODERS,
-    HF_ENCODERS_SHORT,
     image_feature,
     LocalTestBackend,
     number_feature,
     run_experiment,
     sequence_feature,
     set_feature,
-    slow,
     text_feature,
     timeseries_feature,
     vector_feature,
@@ -85,21 +82,10 @@ def run_experiment_with_encoder(encoder, csv_filename):
     run_experiment(input_features, output_features, dataset=rel_path)
 
 
-@pytest.mark.parametrize("encoder", HF_ENCODERS_SHORT)
-def test_experiment_text_feature_HF(encoder, csv_filename):
-    run_experiment_with_encoder(encoder, csv_filename)
-
-
-@slow
-@pytest.mark.parametrize("encoder", HF_ENCODERS)
-def test_experiment_text_feature_HF_full(encoder, csv_filename):
-    run_experiment_with_encoder(encoder, csv_filename)
-
-
 @pytest.mark.parametrize("encoder", ENCODERS)
 def test_experiment_seq_seq_generator(csv_filename, encoder):
     input_features = [text_feature(encoder={"type": encoder, "reduce_output": None})]
-    output_features = [text_feature(decoder={"type": "generator"})]
+    output_features = [text_feature(decoder={"type": "generator"}, output_feature=True)]
     rel_path = generate_data(input_features, output_features, csv_filename)
 
     run_experiment(input_features, output_features, dataset=rel_path)
@@ -108,7 +94,7 @@ def test_experiment_seq_seq_generator(csv_filename, encoder):
 @pytest.mark.parametrize("encoder", ["embed", "rnn", "parallel_cnn", "stacked_parallel_cnn", "transformer"])
 def test_experiment_seq_seq_tagger(csv_filename, encoder):
     input_features = [text_feature(encoder={"type": encoder, "reduce_output": None})]
-    output_features = [text_feature(decoder={"type": "tagger"})]
+    output_features = [text_feature(decoder={"type": "tagger"}, reduce_input=None)]
     rel_path = generate_data(input_features, output_features, csv_filename)
 
     run_experiment(input_features, output_features, dataset=rel_path)
@@ -117,7 +103,7 @@ def test_experiment_seq_seq_tagger(csv_filename, encoder):
 @pytest.mark.parametrize("encoder", ["cnnrnn", "stacked_cnn"])
 def test_experiment_seq_seq_tagger_fails_for_non_length_preserving_encoders(csv_filename, encoder):
     input_features = [text_feature(encoder={"type": encoder, "reduce_output": None})]
-    output_features = [text_feature(decoder={"type": "tagger"})]
+    output_features = [text_feature(decoder={"type": "tagger"}, reduce_input=None)]
     rel_path = generate_data(input_features, output_features, csv_filename)
 
     with pytest.raises(ValueError):
@@ -127,14 +113,14 @@ def test_experiment_seq_seq_tagger_fails_for_non_length_preserving_encoders(csv_
 def test_experiment_seq_seq_model_def_file(csv_filename, yaml_filename):
     # seq-to-seq test to use config file instead of dictionary
     input_features = [text_feature(encoder={"reduce_output": None, "type": "embed"})]
-    output_features = [text_feature(decoder={"reduce_input": None, "vocab_size": 3, "type": "tagger"})]
+    output_features = [text_feature(decoder={"vocab_size": 3, "type": "tagger"}, reduce_input=None)]
 
     # Save the config to a yaml file
     config = {
         "input_features": input_features,
         "output_features": output_features,
         "combiner": {"type": "concat", "output_size": 14},
-        TRAINER: {"epochs": 2},
+        TRAINER: {"epochs": 2, BATCH_SIZE: 128},
     }
     with open(yaml_filename, "w") as yaml_out:
         yaml.safe_dump(config, yaml_out)
@@ -146,7 +132,7 @@ def test_experiment_seq_seq_model_def_file(csv_filename, yaml_filename):
 def test_experiment_seq_seq_train_test_valid(tmpdir):
     # seq-to-seq test to use train, test, validation files
     input_features = [text_feature(encoder={"reduce_output": None, "type": "rnn"})]
-    output_features = [text_feature(decoder={"reduce_input": None, "vocab_size": 3, "type": "tagger"})]
+    output_features = [text_feature(decoder={"vocab_size": 3, "type": "tagger"}, reduce_input=None)]
 
     train_csv = generate_data(input_features, output_features, os.path.join(tmpdir, "train.csv"))
     test_csv = generate_data(input_features, output_features, os.path.join(tmpdir, "test.csv"), 20)
@@ -179,8 +165,8 @@ def test_experiment_multi_input_intent_classification(csv_filename, encoder):
 
 
 def test_experiment_with_torch_module_dict_feature_name(csv_filename):
-    input_features = [{"type": "category", "name": "type"}]
-    output_features = [{"type": "category", "name": "to"}]
+    input_features = [category_feature(name="type")]
+    output_features = [category_feature(name="to", output_feature=True)]
     rel_path = generate_data(input_features, output_features, csv_filename)
 
     run_experiment(input_features, output_features, dataset=rel_path)
@@ -224,7 +210,7 @@ def test_experiment_multilabel_with_class_weights(csv_filename):
         # Generator decoder and reduce_input = None
         [
             category_feature(decoder={"vocab_size": 2, "reduce_input": "sum"}),
-            sequence_feature(decoder={"max_len": 5, "reduce_input": None, "type": "generator"}),
+            sequence_feature(decoder={"max_len": 5, "type": "generator"}, reduce_input=None),
             number_feature(normalization="minmax"),
         ],
         # output features with dependencies single dependency
@@ -321,7 +307,6 @@ ImageParams = namedtuple("ImageTestParams", "image_encoder in_memory_flag skip_s
 @pytest.mark.parametrize(
     "image_params",
     [
-        ImageParams("resnet", True, True),
         ImageParams("stacked_cnn", True, True),
         ImageParams("stacked_cnn", False, False),
     ],
@@ -380,7 +365,7 @@ def test_experiment_image_dataset(train_format, train_in_memory, test_format, te
         "output_features": output_features,
         "combiner": {"type": "concat", "output_size": 14},
         "preprocessing": {},
-        TRAINER: {"epochs": 2},
+        TRAINER: {"epochs": 2, BATCH_SIZE: 128},
     }
 
     # create temporary name for train and test data sets
@@ -392,11 +377,17 @@ def test_experiment_image_dataset(train_format, train_in_memory, test_format, te
     config["input_features"][0]["preprocessing"]["in_memory"] = train_in_memory
     training_set_metadata = None
 
+    # define Ludwig model
     backend = LocalTestBackend()
+    model = LudwigModel(
+        config=config,
+        backend=backend,
+    )
+
     if train_format == "hdf5":
         # hdf5 format
         train_set, _, _, training_set_metadata = preprocess_for_training(
-            config,
+            model.config,
             dataset=train_data,
             backend=backend,
         )
@@ -404,14 +395,9 @@ def test_experiment_image_dataset(train_format, train_in_memory, test_format, te
     else:
         train_dataset_to_use = create_data_set_to_use(train_format, train_data)
 
-    # define Ludwig model
-    model = LudwigModel(
-        config=config,
-        backend=backend,
-    )
     model.train(dataset=train_dataset_to_use, training_set_metadata=training_set_metadata)
 
-    model.config["input_features"][0]["preprocessing"]["in_memory"] = test_in_memory
+    model.config_obj.input_features.to_list()[0]["preprocessing"]["in_memory"] = test_in_memory
 
     # setup test data format to test
     test_data = generate_data(input_features, output_features, test_csv_filename)
@@ -458,14 +444,14 @@ def test_experiment_dataset_formats(data_format, csv_filename):
     # raised for different data set formats and in_memory setting
 
     input_features = [number_feature(), category_feature()]
-    output_features = [category_feature(), number_feature()]
+    output_features = [category_feature(output_feature=True), number_feature()]
 
     config = {
         "input_features": input_features,
         "output_features": output_features,
         "combiner": {"type": "concat", "output_size": 14},
         "preprocessing": {},
-        TRAINER: {"epochs": 2},
+        TRAINER: {"epochs": 2, BATCH_SIZE: 128},
     }
 
     # setup training data format to test
@@ -473,15 +459,16 @@ def test_experiment_dataset_formats(data_format, csv_filename):
 
     training_set_metadata = None
 
+    # define Ludwig model
+    model = LudwigModel(config=config)
+
     if data_format == "hdf5":
         # hdf5 format
-        training_set, _, _, training_set_metadata = preprocess_for_training(config, dataset=raw_data)
+        training_set, _, _, training_set_metadata = preprocess_for_training(model.config, dataset=raw_data)
         dataset_to_use = training_set.data_hdf5_fp
     else:
         dataset_to_use = create_data_set_to_use(data_format, raw_data)
 
-    # define Ludwig model
-    model = LudwigModel(config=config)
     model.train(dataset=dataset_to_use, training_set_metadata=training_set_metadata, random_seed=default_random_seed)
 
     # # run functions with the specified data format
@@ -527,7 +514,7 @@ def test_sequence_tagger(enc_cell_type, attention, csv_filename):
         sequence_feature(encoder={"max_len": 10, "type": "rnn", "cell_type": enc_cell_type, "reduce_output": None})
     ]
     output_features = [
-        sequence_feature(decoder={"max_len": 10, "type": "tagger", "reduce_input": None, "attention": attention})
+        sequence_feature(decoder={"max_len": 10, "type": "tagger", "attention": attention}, reduce_input=None)
     ]
 
     # Generate test data
@@ -540,7 +527,12 @@ def test_sequence_tagger(enc_cell_type, attention, csv_filename):
 def test_sequence_tagger_text(csv_filename):
     # Define input and output features
     input_features = [text_feature(encoder={"max_len": 10, "type": "rnn", "reduce_output": None})]
-    output_features = [sequence_feature(decoder={"max_len": 10, "reduce_input": None, "type": "tagger"})]
+    output_features = [
+        sequence_feature(
+            decoder={"max_len": 10, "type": "tagger"},
+            reduce_input=None,
+        )
+    ]
 
     # Generate test data
     rel_path = generate_data(input_features, output_features, csv_filename)
@@ -575,10 +567,10 @@ def test_experiment_sequence_combiner_with_reduction_fails(csv_filename):
             category_feature(encoder={"vocab_size": 5}),
         ],
         "output_features": [category_feature(decoder={"reduce_input": "sum", "vocab_size": 5})],
-        TRAINER: {"epochs": 2},
+        TRAINER: {"epochs": 2, BATCH_SIZE: 128},
         "combiner": {
             "type": "sequence",
-            "encoder": "rnn",
+            "encoder": {"type": "rnn"},
             "main_sequence_feature": "seq1",
             "reduce_output": None,
         },
@@ -589,7 +581,7 @@ def test_experiment_sequence_combiner_with_reduction_fails(csv_filename):
 
     # Encoding sequence features with 'embed' should fail with SequenceConcatCombiner, since at least one sequence
     # feature should be rank 3.
-    with pytest.raises(ValueError):
+    with pytest.raises(TypeError):
         run_experiment(config=config, dataset=rel_path)
 
 
@@ -620,10 +612,10 @@ def test_experiment_sequence_combiner(sequence_encoder, csv_filename):
             category_feature(vocab_size=5),
         ],
         "output_features": [category_feature(decoder={"reduce_input": "sum", "vocab_size": 5})],
-        TRAINER: {"epochs": 2},
+        TRAINER: {"epochs": 2, BATCH_SIZE: 128},
         "combiner": {
             "type": "sequence",
-            "encoder": "rnn",
+            "encoder": {"type": "rnn"},
             "main_sequence_feature": "seq1",
             "reduce_output": None,
         },
@@ -647,7 +639,7 @@ def test_experiment_model_resume(tmpdir):
         "input_features": input_features,
         "output_features": output_features,
         "combiner": {"type": "concat", "output_size": 14},
-        TRAINER: {"epochs": 2},
+        TRAINER: {"epochs": 2, BATCH_SIZE: 128},
     }
 
     _, _, _, _, output_dir = experiment_cli(config, dataset=rel_path, output_directory=tmpdir)
@@ -658,6 +650,72 @@ def test_experiment_model_resume(tmpdir):
     shutil.rmtree(output_dir, ignore_errors=True)
 
 
+@pytest.mark.distributed
+@pytest.mark.parametrize("dist_strategy", ["horovod", "ddp"])
+def test_experiment_model_resume_distributed(tmpdir, dist_strategy, ray_cluster_4cpu):
+    # Single sequence input, single category output
+    # Tests saving a model file, loading it to rerun training and predict
+    input_features = [number_feature()]
+    output_features = [category_feature(output_feature=True)]
+    # Generate test data
+    rel_path = generate_data(input_features, output_features, os.path.join(tmpdir, "dataset.csv"))
+
+    config = {
+        "input_features": input_features,
+        "output_features": output_features,
+        "combiner": {"type": "concat", "output_size": 8},
+        TRAINER: {"epochs": 1, BATCH_SIZE: 128},
+        "backend": {"type": "ray", "trainer": {"strategy": dist_strategy, "num_workers": 2}},
+    }
+
+    _, _, _, _, output_dir = experiment_cli(config, dataset=rel_path, output_directory=os.path.join(tmpdir, "results1"))
+
+    experiment_cli(
+        config, dataset=rel_path, model_resume_path=output_dir, output_directory=os.path.join(tmpdir, "results2")
+    )
+
+    predict_cli(os.path.join(output_dir, "model"), dataset=rel_path, output_directory=os.path.join(tmpdir, "results3"))
+
+
+@pytest.mark.parametrize(
+    "missing_file",
+    ["training_progress.json", "training_checkpoints"],
+    ids=["training_progress", "training_checkpoints"],
+)
+def test_experiment_model_resume_missing_file(tmpdir, missing_file):
+    # Single sequence input, single category output
+    # Tests saving a model file, loading it to rerun training and predict
+    input_features = [sequence_feature(encoder={"type": "rnn", "reduce_output": "sum"})]
+    output_features = [category_feature(decoder={"reduce_input": "sum", "vocab_size": 2})]
+
+    # Generate test data
+    rel_path = generate_data(input_features, output_features, os.path.join(tmpdir, "dataset.csv"))
+
+    config = {
+        "input_features": input_features,
+        "output_features": output_features,
+        "combiner": {"type": "concat", "output_size": 14},
+        TRAINER: {"epochs": 2, BATCH_SIZE: 128},
+    }
+
+    _, _, _, _, output_dir = experiment_cli(config, dataset=rel_path, output_directory=tmpdir)
+
+    try:
+        # Remove file to simulate failure during first epoch of training which prevents
+        # training_checkpoints to be empty and training_progress.json to not be created
+        missing_file_path = os.path.join(output_dir, "model", missing_file)
+        if missing_file == "training_progress.json":
+            os.remove(missing_file_path)
+        else:
+            shutil.rmtree(missing_file_path)
+    finally:
+        # Training should start a fresh model training run without any errors
+        experiment_cli(config, dataset=rel_path, model_resume_path=output_dir)
+
+    predict_cli(os.path.join(output_dir, "model"), dataset=rel_path)
+    shutil.rmtree(output_dir, ignore_errors=True)
+
+
 def test_experiment_various_feature_types(csv_filename):
     input_features = [binary_feature(), bag_feature()]
     output_features = [set_feature(decoder={"max_len": 3, "vocab_size": 5})]
@@ -682,8 +740,10 @@ def test_visual_question_answering(tmpdir):
     input_features = [
         image_feature(
             folder=image_dest_folder,
-            preprocessing={"in_memory": True, "height": 8, "width": 8, "num_channels": 3, "num_processes": 5},
-            encoder={"type": "resnet", "output_size": 8, "num_filters": 8},
+            preprocessing={"in_memory": True, "height": 32, "width": 32, "num_channels": 3, "num_processes": 5},
+            encoder={
+                "type": "stacked_cnn",
+            },
         ),
         text_feature(encoder={"type": "embed", "min_len": 1}),
     ]
@@ -707,8 +767,10 @@ def test_image_resizing_num_channel_handling(tmpdir):
     input_features = [
         image_feature(
             folder=image_dest_folder,
-            preprocessing={"in_memory": True, "height": 8, "width": 8, "num_channels": 3, "num_processes": 5},
-            encoder={"type": "resnet", "output_size": 8, "num_filters": 8},
+            preprocessing={"in_memory": True, "height": 32, "width": 32, "num_channels": 3, "num_processes": 5},
+            encoder={
+                "type": "stacked_cnn",
+            },
         ),
         text_feature(encoder={"type": "embed", "min_len": 1}),
         number_feature(normalization="minmax"),
@@ -758,7 +820,7 @@ def test_experiment_h3(encoder, csv_filename):
     run_experiment(input_features, output_features, dataset=rel_path)
 
 
-def test_experiment_vector_feature_1(csv_filename):
+def test_experiment_vector_feature(csv_filename):
     input_features = [vector_feature()]
     output_features = [binary_feature()]
     # Generate test data
@@ -767,10 +829,14 @@ def test_experiment_vector_feature_1(csv_filename):
     run_experiment(input_features, output_features, dataset=rel_path)
 
 
-def test_experiment_vector_feature_2(csv_filename):
+def test_experiment_vector_feature_infer_size(csv_filename):
     input_features = [vector_feature()]
     output_features = [vector_feature()]
     # Generate test data
     rel_path = generate_data(input_features, output_features, csv_filename)
 
+    # Unset vector_size so it needs to be inferred
+    del input_features[0][PREPROCESSING]
+    del output_features[0][PREPROCESSING]
+
     run_experiment(input_features, output_features, dataset=rel_path)
