@@ -25,7 +25,7 @@ import torchtext
 
 from ludwig.api import LudwigModel
 from ludwig.backend import RAY
-from ludwig.constants import COMBINER, LOGITS, NAME, PREDICTIONS, PROBABILITIES, TRAINER
+from ludwig.constants import BATCH_SIZE, COMBINER, LOGITS, NAME, PREDICTIONS, PROBABILITIES, TRAINER
 from ludwig.data.preprocessing import preprocess_for_prediction
 from ludwig.features.number_feature import numeric_transformation_registry
 from ludwig.globals import TRAIN_SET_METADATA_FILE_NAME
@@ -118,7 +118,9 @@ def test_torchscript(tmpdir, csv_filename, should_load_model, model_type):
     if model_type == "ecd":
         config[TRAINER] = {"epochs": 2}
     else:
-        config[TRAINER] = {"num_boost_round": 2}
+        # Disable feature filtering to avoid having no features due to small test dataset,
+        # see https://stackoverflow.com/a/66405983/5222402
+        config[TRAINER] = {"num_boost_round": 2, "feature_pre_filter": False}
     ludwig_model = LudwigModel(config, backend=backend)
     ludwig_model.train(
         dataset=data_csv_path,
@@ -174,7 +176,7 @@ def test_torchscript(tmpdir, csv_filename, should_load_model, model_type):
     training_set_metadata_json_fp = os.path.join(ludwigmodel_path, TRAIN_SET_METADATA_FILE_NAME)
 
     dataset, training_set_metadata = preprocess_for_prediction(
-        ludwig_model.config,
+        ludwig_model.config_obj.to_dict(),
         dataset=data_csv_path,
         training_set_metadata=training_set_metadata_json_fp,
         include_outputs=False,
@@ -218,15 +220,17 @@ def test_torchscript(tmpdir, csv_filename, should_load_model, model_type):
 def test_torchscript_e2e_tabular(csv_filename, tmpdir):
     data_csv_path = os.path.join(tmpdir, csv_filename)
     # Configure features to be tested:
-    bin_str_feature = binary_feature()
+    bin_str_feature_input_feature = binary_feature()
+    bin_str_feature_output_feature = binary_feature(output_feature=True)
     transformed_number_features = [
         number_feature(preprocessing={"normalization": numeric_transformer})
         for numeric_transformer in numeric_transformation_registry.keys()
     ]
     input_features = [
-        bin_str_feature,
+        bin_str_feature_input_feature,
         binary_feature(),
         *transformed_number_features,
+        number_feature(preprocessing={"outlier_strategy": "fill_with_mean"}),
         category_feature(encoder={"vocab_size": 3}),
         bag_feature(encoder={"vocab_size": 3}),
         set_feature(encoder={"vocab_size": 3}),
@@ -236,8 +240,8 @@ def test_torchscript_e2e_tabular(csv_filename, tmpdir):
         # h3_feature(),
     ]
     output_features = [
-        bin_str_feature,
-        binary_feature(),
+        bin_str_feature_output_feature,
+        binary_feature(output_feature=True),
         number_feature(),
         category_feature(decoder={"vocab_size": 3}),
         set_feature(decoder={"vocab_size": 3}),
@@ -246,7 +250,11 @@ def test_torchscript_e2e_tabular(csv_filename, tmpdir):
         text_feature(decoder={"vocab_size": 3}),
     ]
     backend = LocalTestBackend()
-    config = {"input_features": input_features, "output_features": output_features, TRAINER: {"epochs": 2}}
+    config = {
+        "input_features": input_features,
+        "output_features": output_features,
+        TRAINER: {"epochs": 2, BATCH_SIZE: 128},
+    }
 
     # Generate training data
     training_data_csv_path = generate_data(input_features, output_features, data_csv_path)
@@ -254,7 +262,12 @@ def test_torchscript_e2e_tabular(csv_filename, tmpdir):
     # Convert bool values to strings, e.g., {'Yes', 'No'}
     df = pd.read_csv(training_data_csv_path)
     false_value, true_value = "No", "Yes"
-    df[bin_str_feature[NAME]] = df[bin_str_feature[NAME]].map(lambda x: true_value if x else false_value)
+    df[bin_str_feature_input_feature[NAME]] = df[bin_str_feature_input_feature[NAME]].map(
+        lambda x: true_value if x else false_value
+    )
+    df[bin_str_feature_output_feature[NAME]] = df[bin_str_feature_output_feature[NAME]].map(
+        lambda x: true_value if x else false_value
+    )
     df.to_csv(training_data_csv_path)
 
     validate_torchscript_outputs(tmpdir, config, backend, training_data_csv_path)
@@ -270,7 +283,11 @@ def test_torchscript_e2e_binary_only(csv_filename, tmpdir):
         binary_feature(),
     ]
     backend = LocalTestBackend()
-    config = {"input_features": input_features, "output_features": output_features, TRAINER: {"epochs": 2}}
+    config = {
+        "input_features": input_features,
+        "output_features": output_features,
+        TRAINER: {"epochs": 2, BATCH_SIZE: 128},
+    }
 
     # Generate training data
     training_data_csv_path = generate_data(input_features, output_features, data_csv_path)
@@ -302,7 +319,7 @@ def test_torchscript_e2e_tabnet_combiner(csv_filename, tmpdir):
             "num_total_blocks": 2,
             "num_shared_blocks": 2,
         },
-        TRAINER: {"epochs": 2},
+        TRAINER: {"epochs": 2, BATCH_SIZE: 128},
     }
 
     # Generate training data
@@ -322,7 +339,11 @@ def test_torchscript_e2e_audio(csv_filename, tmpdir):
         binary_feature(),
     ]
     backend = LocalTestBackend()
-    config = {"input_features": input_features, "output_features": output_features, TRAINER: {"epochs": 2}}
+    config = {
+        "input_features": input_features,
+        "output_features": output_features,
+        TRAINER: {"epochs": 2, BATCH_SIZE: 128},
+    }
     training_data_csv_path = generate_data(input_features, output_features, data_csv_path)
 
     # NOTE: audio preprocessing mismatches by very small margins ~O(1e-6) but causes flakiness in e2e test.
@@ -331,17 +352,28 @@ def test_torchscript_e2e_audio(csv_filename, tmpdir):
     validate_torchscript_outputs(tmpdir, config, backend, training_data_csv_path, tolerance=1e-6)
 
 
-def test_torchscript_e2e_image(tmpdir, csv_filename):
+@pytest.mark.parametrize(
+    "kwargs",
+    [
+        {"encoder": {"type": "stacked_cnn"}},  # Ludwig custom encoder
+        {"encoder": {"type": "alexnet", "use_pretrained": False}},  # TorchVisio pretrained model encoder
+    ],
+)
+def test_torchscript_e2e_image(tmpdir, csv_filename, kwargs):
     data_csv_path = os.path.join(tmpdir, csv_filename)
     image_dest_folder = os.path.join(tmpdir, "generated_images")
     input_features = [
-        image_feature(image_dest_folder),
+        image_feature(image_dest_folder, **kwargs),
     ]
     output_features = [
         binary_feature(),
     ]
     backend = LocalTestBackend()
-    config = {"input_features": input_features, "output_features": output_features, TRAINER: {"epochs": 2}}
+    config = {
+        "input_features": input_features,
+        "output_features": output_features,
+        TRAINER: {"epochs": 2, BATCH_SIZE: 128},
+    }
     training_data_csv_path = generate_data(input_features, output_features, data_csv_path)
 
     validate_torchscript_outputs(tmpdir, config, backend, training_data_csv_path)
@@ -357,15 +389,19 @@ def test_torchscript_e2e_text(tmpdir, csv_filename):
         text_feature(decoder={"vocab_size": 3}),
     ]
     backend = LocalTestBackend()
-    config = {"input_features": input_features, "output_features": output_features, TRAINER: {"epochs": 2}}
+    config = {
+        "input_features": input_features,
+        "output_features": output_features,
+        TRAINER: {"epochs": 2, BATCH_SIZE: 128},
+    }
     training_data_csv_path = generate_data(input_features, output_features, data_csv_path)
 
     validate_torchscript_outputs(tmpdir, config, backend, training_data_csv_path)
 
 
 @pytest.mark.skipif(
-    torch.torch_version.TorchVersion(torchtext.__version__) < (0, 13, 0),
-    reason="requires torchtext 0.13.0 or higher",
+    torch.torch_version.TorchVersion(torchtext.__version__) < (0, 14, 0),
+    reason="requires torchtext 0.14.0 or higher",
 )
 def test_torchscript_e2e_text_hf_tokenizer(tmpdir, csv_filename):
     data_csv_path = os.path.join(tmpdir, csv_filename)
@@ -374,7 +410,32 @@ def test_torchscript_e2e_text_hf_tokenizer(tmpdir, csv_filename):
         text_feature(decoder={"vocab_size": 3}),
     ]
     backend = LocalTestBackend()
-    config = {"input_features": input_features, "output_features": output_features, TRAINER: {"epochs": 2}}
+    config = {
+        "input_features": input_features,
+        "output_features": output_features,
+        TRAINER: {"epochs": 2, BATCH_SIZE: 128},
+    }
+    training_data_csv_path = generate_data(input_features, output_features, data_csv_path)
+
+    validate_torchscript_outputs(tmpdir, config, backend, training_data_csv_path)
+
+
+@pytest.mark.skipif(
+    torch.torch_version.TorchVersion(torchtext.__version__) < (0, 14, 0),
+    reason="requires torchtext 0.14.0 or higher",
+)
+def test_torchscript_e2e_text_hf_tokenizer_truncated_sequence(tmpdir, csv_filename):
+    data_csv_path = os.path.join(tmpdir, csv_filename)
+    input_features = [text_feature(encoder={"vocab_size": 3, "type": "bert"}, preprocessing={"max_sequence_length": 3})]
+    output_features = [
+        text_feature(decoder={"vocab_size": 3}),
+    ]
+    backend = LocalTestBackend()
+    config = {
+        "input_features": input_features,
+        "output_features": output_features,
+        TRAINER: {"epochs": 2, BATCH_SIZE: 128},
+    }
     training_data_csv_path = generate_data(input_features, output_features, data_csv_path)
 
     validate_torchscript_outputs(tmpdir, config, backend, training_data_csv_path)
@@ -389,7 +450,11 @@ def test_torchscript_e2e_sequence(tmpdir, csv_filename):
         sequence_feature(decoder={"vocab_size": 3}),
     ]
     backend = LocalTestBackend()
-    config = {"input_features": input_features, "output_features": output_features, TRAINER: {"epochs": 2}}
+    config = {
+        "input_features": input_features,
+        "output_features": output_features,
+        TRAINER: {"epochs": 2, BATCH_SIZE: 128},
+    }
     training_data_csv_path = generate_data(input_features, output_features, data_csv_path)
 
     validate_torchscript_outputs(tmpdir, config, backend, training_data_csv_path)
@@ -404,7 +469,11 @@ def test_torchscript_e2e_timeseries(tmpdir, csv_filename):
         binary_feature(),
     ]
     backend = LocalTestBackend()
-    config = {"input_features": input_features, "output_features": output_features, TRAINER: {"epochs": 2}}
+    config = {
+        "input_features": input_features,
+        "output_features": output_features,
+        TRAINER: {"epochs": 2, BATCH_SIZE: 128},
+    }
     training_data_csv_path = generate_data(input_features, output_features, data_csv_path)
 
     validate_torchscript_outputs(tmpdir, config, backend, training_data_csv_path)
@@ -419,7 +488,11 @@ def test_torchscript_e2e_h3(tmpdir, csv_filename):
         binary_feature(),
     ]
     backend = LocalTestBackend()
-    config = {"input_features": input_features, "output_features": output_features, TRAINER: {"epochs": 2}}
+    config = {
+        "input_features": input_features,
+        "output_features": output_features,
+        TRAINER: {"epochs": 2, BATCH_SIZE: 128},
+    }
     training_data_csv_path = generate_data(input_features, output_features, data_csv_path)
 
     validate_torchscript_outputs(tmpdir, config, backend, training_data_csv_path)
@@ -434,7 +507,11 @@ def test_torchscript_e2e_date(tmpdir, csv_filename):
         binary_feature(),
     ]
     backend = LocalTestBackend()
-    config = {"input_features": input_features, "output_features": output_features, TRAINER: {"epochs": 2}}
+    config = {
+        "input_features": input_features,
+        "output_features": output_features,
+        TRAINER: {"epochs": 2, BATCH_SIZE: 128},
+    }
     training_data_csv_path = generate_data(input_features, output_features, data_csv_path)
 
     validate_torchscript_outputs(tmpdir, config, backend, training_data_csv_path)
@@ -451,7 +528,11 @@ def test_torchscript_preproc_vector_alternative_type(tmpdir, csv_filename, vecto
         binary_feature(),
     ]
     backend = LocalTestBackend()
-    config = {"input_features": input_features, "output_features": output_features, TRAINER: {"epochs": 2}}
+    config = {
+        "input_features": input_features,
+        "output_features": output_features,
+        TRAINER: {"epochs": 2, BATCH_SIZE: 128},
+    }
     training_data_csv_path = generate_data(input_features, output_features, data_csv_path)
 
     # Initialize Ludwig model
@@ -459,7 +540,7 @@ def test_torchscript_preproc_vector_alternative_type(tmpdir, csv_filename, vecto
 
     # Obtain preprocessed inputs from Python model
     preproc_inputs_expected, _ = preprocess_for_prediction(
-        ludwig_model.config,
+        ludwig_model.config_obj.to_dict(),
         training_data_csv_path,
         ludwig_model.training_set_metadata,
         backend=backend,
@@ -511,7 +592,11 @@ def test_torchscript_preproc_timeseries_alternative_type(tmpdir, csv_filename, p
         binary_feature(),
     ]
     backend = LocalTestBackend()
-    config = {"input_features": input_features, "output_features": output_features, TRAINER: {"epochs": 2}}
+    config = {
+        "input_features": input_features,
+        "output_features": output_features,
+        TRAINER: {"epochs": 2, BATCH_SIZE: 128},
+    }
     training_data_csv_path = generate_data(input_features, output_features, data_csv_path, nan_percent=0.2)
 
     # Initialize Ludwig model
@@ -519,7 +604,7 @@ def test_torchscript_preproc_timeseries_alternative_type(tmpdir, csv_filename, p
 
     # Obtain preprocessed inputs from Python model
     preproc_inputs_expected, _ = preprocess_for_prediction(
-        ludwig_model.config,
+        ludwig_model.config_obj.to_dict(),
         training_data_csv_path,
         ludwig_model.training_set_metadata,
         backend=backend,
@@ -576,7 +661,11 @@ def test_torchscript_preproc_with_nans(tmpdir, csv_filename, feature):
         binary_feature(),
     ]
     backend = LocalTestBackend()
-    config = {"input_features": input_features, "output_features": output_features, TRAINER: {"epochs": 2}}
+    config = {
+        "input_features": input_features,
+        "output_features": output_features,
+        TRAINER: {"epochs": 2, BATCH_SIZE: 128},
+    }
     training_data_csv_path = generate_data(input_features, output_features, data_csv_path, nan_percent=0.2)
 
     # Initialize Ludwig model
@@ -584,7 +673,7 @@ def test_torchscript_preproc_with_nans(tmpdir, csv_filename, feature):
 
     # Obtain preprocessed inputs from Python model
     preproc_inputs_expected, _ = preprocess_for_prediction(
-        ludwig_model.config,
+        ludwig_model.config_obj.to_dict(),
         training_data_csv_path,
         ludwig_model.training_set_metadata,
         backend=backend,
@@ -642,7 +731,11 @@ def test_torchscript_preproc_gpu(tmpdir, csv_filename, feature_fn):
         binary_feature(),
     ]
 
-    config = {"input_features": input_features, "output_features": output_features, TRAINER: {"epochs": 2}}
+    config = {
+        "input_features": input_features,
+        "output_features": output_features,
+        TRAINER: {"epochs": 2, BATCH_SIZE: 128},
+    }
     backend = RAY
     training_data_csv_path = generate_data(input_features, output_features, data_csv_path)
     _, script_module = initialize_torchscript_module(
@@ -695,7 +788,11 @@ def test_torchscript_postproc_gpu(tmpdir, csv_filename, feature_fn):
         feature_fn(**feature_kwargs),
     ]
 
-    config = {"input_features": input_features, "output_features": output_features, TRAINER: {"epochs": 2}}
+    config = {
+        "input_features": input_features,
+        "output_features": output_features,
+        TRAINER: {"epochs": 2, BATCH_SIZE: 128},
+    }
     backend = RAY
     training_data_csv_path = generate_data(input_features, output_features, data_csv_path)
     _, script_module = initialize_torchscript_module(
