@@ -22,10 +22,9 @@ import shutil
 import sys
 import tempfile
 import traceback
-import unittest
 import uuid
 from distutils.util import strtobool
-from typing import List, Union
+from typing import Any, Dict, List, Optional, Set, Tuple, TYPE_CHECKING, Union
 
 import cloudpickle
 import numpy as np
@@ -36,14 +35,41 @@ from PIL import Image
 
 from ludwig.api import LudwigModel
 from ludwig.backend import LocalBackend
-from ludwig.constants import COLUMN, DECODER, ENCODER, NAME, PROC_COLUMN, TRAINER, VECTOR
+from ludwig.constants import (
+    AUDIO,
+    BAG,
+    BATCH_SIZE,
+    BINARY,
+    CATEGORY,
+    COLUMN,
+    DATE,
+    DECODER,
+    ENCODER,
+    H3,
+    IMAGE,
+    NAME,
+    NUMBER,
+    PROC_COLUMN,
+    SEQUENCE,
+    SET,
+    SPLIT,
+    TEXT,
+    TIMESERIES,
+    TRAINER,
+    VECTOR,
+)
 from ludwig.data.dataset_synthesizer import build_synthetic_dataset, DATETIME_FORMATS
 from ludwig.experiment import experiment_cli
 from ludwig.features.feature_utils import compute_feature_hash
+from ludwig.globals import PREDICTIONS_PARQUET_FILE_NAME
 from ludwig.trainers.trainer import Trainer
 from ludwig.utils import fs_utils
 from ludwig.utils.data_utils import read_csv, replace_file_extension, use_credentials
 
+if TYPE_CHECKING:
+    from ludwig.data.dataset.base import Dataset
+    from ludwig.schema.model_types.base import ModelConfig
+
 logger = logging.getLogger(__name__)
 
 # Used in sequence-related unit tests (encoders, features) as well as end-to-end integration tests.
@@ -56,12 +82,12 @@ HF_ENCODERS = [
     "bert",
     "gpt",
     "gpt2",
-    # 'transformer_xl',
+    "transformer_xl",
     "xlnet",
-    "xlm",
+    # "xlm",  # disabled in the schema: https://github.com/ludwig-ai/ludwig/pull/3108
     "roberta",
     "distilbert",
-    "ctrl",
+    # "ctrl",  # disabled in the schema: https://github.com/ludwig-ai/ludwig/pull/2976
     "camembert",
     "albert",
     "t5",
@@ -69,7 +95,7 @@ HF_ENCODERS = [
     "longformer",
     "flaubert",
     "electra",
-    "mt5",
+    # "mt5",    # disabled in the schema: https://github.com/ludwig-ai/ludwig/pull/2982
 ]
 
 RAY_BACKEND_CONFIG = {
@@ -79,7 +105,7 @@ RAY_BACKEND_CONFIG = {
     },
     "trainer": {
         "use_gpu": False,
-        "num_workers": 2,
+        "num_workers": 1,
         "resources_per_worker": {
             "CPU": 0.1,
             "GPU": 0,
@@ -126,20 +152,9 @@ def parse_flag_from_env(key, default=False):
     return _value
 
 
-_run_slow_tests = parse_flag_from_env("RUN_SLOW", default=False)
 _run_private_tests = parse_flag_from_env("RUN_PRIVATE", default=False)
 
 
-def slow(test_case):
-    """Decorator marking a test as slow.
-
-    Slow tests are skipped by default. Set the RUN_SLOW environment variable to a truth value to run them.
-    """
-    if not _run_slow_tests:
-        test_case = unittest.skip("Skipping: this test is too slow")(test_case)
-    return test_case
-
-
 def private_param(param):
     """Wrap param to mark it as private, meaning it requires credentials to run.
 
@@ -170,16 +185,32 @@ def generate_data(
     :param nan_percent: percent of values in a feature to be NaN
     :return:
     """
+    df = generate_data_as_dataframe(input_features, output_features, num_examples, nan_percent)
+    df.to_csv(filename, index=False)
+    return filename
+
+
+def generate_data_as_dataframe(
+    input_features,
+    output_features,
+    num_examples=25,
+    nan_percent=0.0,
+) -> pd.DataFrame:
+    """Helper method to generate synthetic data based on input, output feature specs.
+
+    Args:
+        num_examples: number of examples to generate
+        input_features: schema
+        output_features: schema
+        nan_percent: percent of values in a feature to be NaN
+    Returns:
+        A pandas DataFrame
+    """
     features = input_features + output_features
     df = build_synthetic_dataset(num_examples, features)
     data = [next(df) for _ in range(num_examples + 1)]
 
-    dataframe = pd.DataFrame(data[1:], columns=data[0])
-    if nan_percent > 0:
-        add_nans_to_df_in_place(dataframe, nan_percent)
-    dataframe.to_csv(filename, index=False)
-
-    return filename
+    return pd.DataFrame(data[1:], columns=data[0])
 
 
 def recursive_update(dictionary, values):
@@ -197,8 +228,8 @@ def random_string(length=5):
 
 def number_feature(normalization=None, **kwargs):
     feature = {
-        "name": "num_" + random_string(),
-        "type": "number",
+        "name": f"{NUMBER}_{random_string()}",
+        "type": NUMBER,
         "preprocessing": {"normalization": normalization},
     }
     recursive_update(feature, kwargs)
@@ -211,8 +242,8 @@ def category_feature(output_feature=False, **kwargs):
     if DECODER in kwargs:
         output_feature = True
     feature = {
-        "type": "category",
-        "name": "category_" + random_string(),
+        "name": f"{CATEGORY}_{random_string()}",
+        "type": CATEGORY,
     }
     if output_feature:
         feature.update(
@@ -223,7 +254,7 @@ def category_feature(output_feature=False, **kwargs):
     else:
         feature.update(
             {
-                ENCODER: {"type": "dense", "vocab_size": 10, "embedding_size": 5},
+                ENCODER: {"vocab_size": 10, "embedding_size": 5},
             }
         )
     recursive_update(feature, kwargs)
@@ -236,8 +267,8 @@ def text_feature(output_feature=False, **kwargs):
     if DECODER in kwargs:
         output_feature = True
     feature = {
-        "name": "text_" + random_string(),
-        "type": "text",
+        "name": f"{TEXT}_{random_string()}",
+        "type": TEXT,
     }
     if output_feature:
         feature.update(
@@ -268,8 +299,8 @@ def set_feature(output_feature=False, **kwargs):
     if DECODER in kwargs:
         output_feature = True
     feature = {
-        "type": "set",
-        "name": "set_" + random_string(),
+        "name": f"{SET}_{random_string()}",
+        "type": SET,
     }
     if output_feature:
         feature.update(
@@ -293,8 +324,8 @@ def sequence_feature(output_feature=False, **kwargs):
     if DECODER in kwargs:
         output_feature = True
     feature = {
-        "type": "sequence",
-        "name": "sequence_" + random_string(),
+        "name": f"{SEQUENCE}_{random_string()}",
+        "type": SEQUENCE,
     }
     if output_feature:
         feature.update(
@@ -329,14 +360,11 @@ def sequence_feature(output_feature=False, **kwargs):
 
 def image_feature(folder, **kwargs):
     feature = {
-        "type": "image",
-        "name": "image_" + random_string(),
+        "name": f"{IMAGE}_{random_string()}",
+        "type": IMAGE,
         "preprocessing": {"in_memory": True, "height": 12, "width": 12, "num_channels": 3},
         ENCODER: {
-            "type": "resnet",
-            "resnet_size": 8,
-            "num_filters": 8,
-            "output_size": 8,
+            "type": "stacked_cnn",
         },
         "destination_folder": folder,
     }
@@ -348,8 +376,8 @@ def image_feature(folder, **kwargs):
 
 def audio_feature(folder, **kwargs):
     feature = {
-        "name": "audio_" + random_string(),
-        "type": "audio",
+        "name": f"{AUDIO}_{random_string()}",
+        "type": AUDIO,
         "preprocessing": {
             "type": "fbank",
             "window_length_in_s": 0.04,
@@ -376,8 +404,8 @@ def audio_feature(folder, **kwargs):
 
 def timeseries_feature(**kwargs):
     feature = {
-        "name": "timeseries_" + random_string(),
-        "type": "timeseries",
+        "name": f"{TIMESERIES}_{random_string()}",
+        "type": TIMESERIES,
         ENCODER: {"type": "parallel_cnn", "max_len": 7},
     }
     recursive_update(feature, kwargs)
@@ -388,8 +416,8 @@ def timeseries_feature(**kwargs):
 
 def binary_feature(**kwargs):
     feature = {
-        "name": "binary_" + random_string(),
-        "type": "binary",
+        "name": f"{BINARY}_{random_string()}",
+        "type": BINARY,
     }
     recursive_update(feature, kwargs)
     feature[COLUMN] = feature[NAME]
@@ -399,8 +427,8 @@ def binary_feature(**kwargs):
 
 def bag_feature(**kwargs):
     feature = {
-        "name": "bag_" + random_string(),
-        "type": "bag",
+        "name": f"{BAG}_{random_string()}",
+        "type": BAG,
         ENCODER: {"type": "embed", "max_len": 5, "vocab_size": 10, "embedding_size": 5},
     }
     recursive_update(feature, kwargs)
@@ -411,9 +439,11 @@ def bag_feature(**kwargs):
 
 def date_feature(**kwargs):
     feature = {
-        "name": "date_" + random_string(),
-        "type": "date",
-        "preprocessing": {"datetime_format": random.choice(list(DATETIME_FORMATS.keys()))},
+        "name": f"{DATE}_{random_string()}",
+        "type": DATE,
+        "preprocessing": {
+            "datetime_format": random.choice(list(DATETIME_FORMATS.keys())),
+        },
     }
     recursive_update(feature, kwargs)
     feature[COLUMN] = feature[NAME]
@@ -422,7 +452,10 @@ def date_feature(**kwargs):
 
 
 def h3_feature(**kwargs):
-    feature = {"name": "h3_" + random_string(), "type": "h3"}
+    feature = {
+        "name": f"{H3}_{random_string()}",
+        "type": H3,
+    }
     recursive_update(feature, kwargs)
     feature[COLUMN] = feature[NAME]
     feature[PROC_COLUMN] = compute_feature_hash(feature)
@@ -431,8 +464,8 @@ def h3_feature(**kwargs):
 
 def vector_feature(**kwargs):
     feature = {
+        "name": f"{VECTOR}_{random_string()}",
         "type": VECTOR,
-        "name": "vector_" + random_string(),
         "preprocessing": {
             "vector_size": 5,
         },
@@ -463,7 +496,7 @@ def run_experiment(
             "input_features": input_features,
             "output_features": output_features,
             "combiner": {"type": "concat", "output_size": 14},
-            TRAINER: {"epochs": 2},
+            TRAINER: {"epochs": 2, BATCH_SIZE: 128},
         }
 
     with tempfile.TemporaryDirectory() as tmpdir:
@@ -500,8 +533,8 @@ def generate_output_features_with_dependencies(main_feature, dependencies):
     """
 
     output_features = [
-        category_feature(decoder={"type": "classifier", "vocab_size": 2}, reduce_input="sum"),
-        sequence_feature(decoder={"type": "generator", "vocab_size": 10, "max_len": 5}),
+        category_feature(decoder={"type": "classifier", "vocab_size": 2}, reduce_input="sum", output_feature=True),
+        sequence_feature(decoder={"type": "generator", "vocab_size": 10, "max_len": 5}, output_feature=True),
         number_feature(),
     ]
 
@@ -536,8 +569,8 @@ def generate_output_features_with_dependencies_complex():
     )
 
     # The correct order ids[tf, sf, nf, vf, set_f, cf]
-    # # shuffling it to test the robustness of the topological sort
-    output_features = [nf, tf, set_f, vf, cf, sf, nf]
+    # shuffling it to test the robustness of the topological sort
+    output_features = [nf, tf, set_f, vf, cf, sf]
 
     return output_features
 
@@ -624,7 +657,7 @@ def run_api_experiment(input_features, output_features, data_csv):
         "input_features": input_features,
         "output_features": output_features,
         "combiner": {"type": "concat", "output_size": 14},
-        TRAINER: {"epochs": 2},
+        TRAINER: {"epochs": 2, BATCH_SIZE: 128},
     }
 
     model = LudwigModel(config)
@@ -673,6 +706,8 @@ def add_nans_to_df_in_place(df: pd.DataFrame, nan_percent: float):
     num_rows = len(df)
     num_nans_per_col = int(round(nan_percent * num_rows))
     for col in df.columns:
+        if col == SPLIT:  # do not add NaNs to the split column
+            continue
         col_idx = df.columns.get_loc(col)
         for row_idx in random.sample(range(num_rows), num_nans_per_col):
             df.iloc[row_idx, col_idx] = np.nan
@@ -779,7 +814,7 @@ def create_data_set_to_use(data_format, raw_data, nan_percent=0.0):
 
 
 def augment_dataset_with_none(
-    df: pd.DataFrame, first_row_none: bool = False, last_row_none: bool = False, nan_cols: List = []
+    df: pd.DataFrame, first_row_none: bool = False, last_row_none: bool = False, nan_cols: Optional[List] = None
 ) -> pd.DataFrame:
     """Optionally sets the first and last rows of nan_cols of the given dataframe to nan.
 
@@ -792,6 +827,8 @@ def augment_dataset_with_none(
     :param nan_cols: a list of columns in the dataframe to explicitly set the first or last rows to np.nan
     :type nan_cols: list
     """
+    nan_cols = nan_cols if nan_cols is not None else []
+
     if first_row_none:
         for col in nan_cols:
             df.iloc[0, df.columns.get_loc(col)] = np.nan
@@ -813,12 +850,11 @@ def train_with_backend(
     callbacks=None,
     skip_save_processed_input=True,
     skip_save_predictions=True,
+    required_metrics=None,
 ):
     model = LudwigModel(config, backend=backend, callbacks=callbacks)
-    output_dir = None
-
-    try:
-        _, _, output_dir = model.train(
+    with tempfile.TemporaryDirectory() as output_directory:
+        _, _, _ = model.train(
             dataset=dataset,
             training_set=training_set,
             validation_set=validation_set,
@@ -827,20 +863,32 @@ def train_with_backend(
             skip_save_progress=True,
             skip_save_unprocessed_output=True,
             skip_save_log=True,
+            output_directory=output_directory,
         )
 
         if dataset is None:
             dataset = training_set
 
         if predict:
-            preds, _ = model.predict(dataset=dataset, skip_save_predictions=skip_save_predictions)
+            preds, _ = model.predict(
+                dataset=dataset, skip_save_predictions=skip_save_predictions, output_directory=output_directory
+            )
             assert preds is not None
 
+            if not skip_save_predictions:
+                read_preds = model.backend.df_engine.read_predictions(
+                    os.path.join(output_directory, PREDICTIONS_PARQUET_FILE_NAME)
+                )
+                # call compute to ensure preds materialize correctly
+                read_preds = read_preds.compute()
+                assert read_preds is not None
+
         if evaluate:
             eval_stats, eval_preds, _ = model.evaluate(
                 dataset=dataset, collect_overall_stats=False, collect_predictions=True
             )
             assert eval_preds is not None
+            assert_all_required_metrics_exist(eval_stats, required_metrics)
 
             # Test that eval_stats are approx equal when using local backend
             with tempfile.TemporaryDirectory() as tmpdir:
@@ -857,23 +905,121 @@ def train_with_backend(
                         k: {
                             metric_name: value
                             for metric_name, value in v.items()
-                            if metric_name not in {"loss", "root_mean_squared_percentage_error"}
+                            if metric_name not in {"loss", "root_mean_squared_percentage_error", "jaccard"}
                         }
                         for k, v in stats.items()
                     }
 
-                for (k1, v1), (k2, v2) in zip(filter(eval_stats).items(), filter(local_eval_stats).items()):
-                    assert k1 == k2
-                    for (name1, metric1), (name2, metric2) in zip(v1.items(), v2.items()):
-                        assert name1 == name2
-                        assert np.isclose(
-                            metric1, metric2, rtol=1e-04, atol=1e-5
-                        ), f"metric {name1}: {metric1} != {metric2}"
+                for (feature_name_from_eval, metrics_dict_from_eval), (
+                    feature_name_from_local,
+                    metrics_dict_from_local,
+                ) in zip(filter(eval_stats).items(), filter(local_eval_stats).items()):
+                    for (metric_name_from_eval, metric_value_from_eval), (
+                        metric_name_from_local,
+                        metric_value_from_local,
+                    ) in zip(metrics_dict_from_eval.items(), metrics_dict_from_local.items()):
+                        assert metric_name_from_eval == metric_name_from_local, (
+                            f"Metric mismatch between eval and local. Metrics from eval: "
+                            f"{metrics_dict_from_eval.keys()}. Metrics from local: {metrics_dict_from_local.keys()}"
+                        )
+                        assert np.isclose(metric_value_from_eval, metric_value_from_local, rtol=1e-03, atol=1e-04), (
+                            f"Metric {metric_name_from_eval} for feature {feature_name_from_eval}: "
+                            f"{metric_value_from_eval} != {metric_value_from_local}"
+                        )
 
         return model
-    finally:
-        # Remove results/intermediate data saved to disk
-        shutil.rmtree(output_dir, ignore_errors=True)
+
+
+def assert_all_required_metrics_exist(
+    feature_to_metrics_dict: Dict[str, Dict[str, Any]], required_metrics: Optional[Dict[str, Set]] = None
+):
+    """Checks that all `required_metrics` exist in the dictionary returned during Ludwig model evaluation.
+
+    `feature_to_metrics_dict` is a dict where the feature name is a key and the value is a dictionary of metrics:
+
+        {
+            "binary_1234": {
+                "accuracy": 0.5,
+                "loss": 0.5,
+            },
+            "numerical_1234": {
+                "mean_squared_error": 0.5,
+                "loss": 0.5,
+            }
+        }
+
+    `required_metrics` is a dict where the feature name is a key and the value is a set of metric names:
+
+        {
+            "binary_1234": {"accuracy"},
+            "numerical_1234": {"mean_squared_error"},
+        }
+
+    Args:
+        feature_to_metrics_dict: dictionary of output feature to a dictionary of metrics
+        required_metrics: optional dictionary of output feature to a set of metrics names. If None, then function
+            returns True immediately.
+    Returns:
+        None. Raises an AssertionError if any required metrics are missing.
+    """
+    if required_metrics is None:
+        return
+
+    for feature_name, metrics_dict in feature_to_metrics_dict.items():
+        if feature_name in required_metrics:
+            required_metric_names = set(required_metrics[feature_name])
+            metric_names = set(metrics_dict.keys())
+            assert required_metric_names.issubset(
+                metric_names
+            ), f"required metrics {required_metric_names} not in metrics {metric_names} for feature {feature_name}"
+
+
+def assert_preprocessed_dataset_shape_and_dtype_for_feature(
+    feature_name: str,
+    preprocessed_dataset: "Dataset",
+    config_obj: "ModelConfig",
+    expected_dtype: np.dtype,
+    expected_shape: Tuple,
+):
+    """Asserts that the preprocessed dataset has the correct shape and dtype for a given feature type.
+
+    Args:
+        feature_name: the name of the feature to check
+        preprocessed_dataset: the preprocessed dataset
+        config_obj: the model config object
+        expected_dtype: the expected dtype
+        expected_shape: the expected shape
+    Returns:
+        None.
+    Raises:
+        AssertionError if the preprocessed dataset does not have the correct shape and dtype for the given feature type.
+    """
+    if_configs = [if_config for if_config in config_obj.input_features if if_config.name == feature_name]
+    # fail fast if given `feature_name`` is not found or is not unique
+    if len(if_configs) != 1:
+        raise ValueError(f"feature_name {feature_name} found {len(if_configs)} times in config_obj")
+    if_config = if_configs[0]
+
+    if_config_proc_column = if_config.proc_column
+    for result in [
+        preprocessed_dataset.training_set,
+        preprocessed_dataset.validation_set,
+        preprocessed_dataset.test_set,
+    ]:
+        result_df = result.to_df()
+        result_df_proc_col = result_df[if_config_proc_column]
+
+        # Check that the proc col is of the correct dtype
+        result_df_proc_col_dtypes = set(result_df_proc_col.map(lambda x: x.dtype))
+        assert all(
+            [expected_dtype == dtype for dtype in result_df_proc_col_dtypes]
+        ), f"proc dtype should be {expected_dtype}, got the following set of values: {result_df_proc_col_dtypes}"
+
+        # Check that the proc col is of the right dimensions
+        result_df_proc_col_shapes = set(result_df_proc_col.map(lambda x: x.shape))
+        assert all(
+            expected_shape == shape for shape in result_df_proc_col_shapes
+        ), f"proc shape should be {expected_shape}, got the following set of values: {result_df_proc_col_shapes}"
 
 
 @contextlib.contextmanager
@@ -894,7 +1040,7 @@ def remote_tmpdir(fs_protocol, bucket):
             with use_credentials(minio_test_creds()):
                 fs_utils.delete(tmpdir, recursive=True)
         except FileNotFoundError as e:
-            logging.info(f"failed to delete remote tempdir, does not exist: {str(e)}")
+            logger.info(f"failed to delete remote tempdir, does not exist: {str(e)}")
             pass
 
 
