@@ -5,7 +5,8 @@ import numpy as np
 import pytest
 
 from ludwig.api import LudwigModel
-from ludwig.constants import TRAINER
+from ludwig.constants import BATCH_SIZE, TRAINER
+from ludwig.utils.numerical_test_utils import assert_all_finite
 from tests.integration_tests.utils import (
     audio_feature,
     bag_feature,
@@ -25,14 +26,20 @@ from tests.integration_tests.utils import (
 
 
 @pytest.mark.distributed
+@pytest.mark.skip(reason="https://github.com/ludwig-ai/ludwig/issues/2686")
 def test_training_determinism_ray_backend(csv_filename, tmpdir, ray_cluster_4cpu):
     experiment_output_1, experiment_output_2 = train_twice("ray", csv_filename, tmpdir)
 
     eval_stats_1, train_stats_1, _, _ = experiment_output_1
     eval_stats_2, train_stats_2, _, _ = experiment_output_2
 
-    np.testing.assert_equal(eval_stats_1, eval_stats_1)
-    np.testing.assert_equal(train_stats_1, train_stats_1)
+    assert_all_finite(eval_stats_1)
+    assert_all_finite(eval_stats_2)
+    assert_all_finite(train_stats_1)
+    assert_all_finite(train_stats_2)
+
+    np.testing.assert_equal(eval_stats_1, eval_stats_2)
+    np.testing.assert_equal(train_stats_1, train_stats_2)
 
 
 def test_training_determinism_local_backend(csv_filename, tmpdir):
@@ -41,8 +48,13 @@ def test_training_determinism_local_backend(csv_filename, tmpdir):
     eval_stats_1, train_stats_1, _, _ = experiment_output_1
     eval_stats_2, train_stats_2, _, _ = experiment_output_2
 
-    np.testing.assert_equal(eval_stats_1, eval_stats_1)
-    np.testing.assert_equal(train_stats_1, train_stats_1)
+    assert_all_finite(eval_stats_1)
+    assert_all_finite(eval_stats_2)
+    assert_all_finite(train_stats_1)
+    assert_all_finite(train_stats_2)
+
+    np.testing.assert_equal(eval_stats_1, eval_stats_2)
+    np.testing.assert_equal(train_stats_1, train_stats_2)
 
 
 def train_twice(backend, csv_filename, tmpdir):
@@ -70,10 +82,14 @@ def train_twice(backend, csv_filename, tmpdir):
         number_feature(),
         category_feature(decoder={"vocab_size": 10}),
     ]
-    config = {"input_features": input_features, "output_features": output_features, TRAINER: {"epochs": 2}}
+    config = {
+        "input_features": input_features,
+        "output_features": output_features,
+        TRAINER: {"epochs": 2, BATCH_SIZE: 128},
+    }
 
     # Generate training data
-    training_data_csv_path = generate_data(input_features, output_features, csv_filename)
+    training_data_csv_path = generate_data(input_features, output_features, csv_filename, num_examples=100)
 
     ludwig_model_1 = LudwigModel(config, logging_level=logging.ERROR, backend=backend)
     ludwig_model_2 = LudwigModel(config, logging_level=logging.ERROR, backend=backend)
